import time
import unittest
from unittest.case import TestCase

from devices.devices import ZlgDevice, DeviceResult
from framework.canfactory import create_batch_from_zcanpro_exporting_file
from framework.canmsg import CanMsgBatch, CanMsg, CanFrame


class TestZlgDevice(TestCase):

    def setUp(self):
        self.dev = ZlgDevice()
        self.dev.open()

    def tearDown(self):
        self.dev.close()

    def test_open_channel(self):
        self.assertEqual(DeviceResult.SUCCESS, self.dev.open_channel())

    @unittest.skip("connect the chn0 to another can end")
    def test_send(self):
        self.assertEqual(DeviceResult.SUCCESS, self.dev.open_channel())
        batch = create_batch_from_zcanpro_exporting_file("files/test_batch_1_msg.list")
        can_msg = batch.can_msgs[0]
        self.assertEqual(1, self.dev.send(can_msg))
        self.assertEqual(DeviceResult.SUCCESS, self.dev.close_channel())

    def test_send_to_myself(self):
        self.assertEqual(DeviceResult.SUCCESS, self.dev.open_channel())
        batch = create_batch_from_zcanpro_exporting_file("files/test_batch_1_msg.list")
        can_msg = batch.can_msgs[0]
        self.assertEqual(1, self.dev.send_to_myself(can_msg))
        self.assertEqual(DeviceResult.SUCCESS, self.dev.close_channel())

    def test_receive(self):
        self.assertEqual(DeviceResult.SUCCESS, self.dev.open_channel())
        batch = create_batch_from_zcanpro_exporting_file("files/test_batch_1_msg.list")
        can_msg = batch.can_msgs[0]
        self.assertEqual(1, self.dev.send_to_myself(can_msg))
        time.sleep(1)
        rec = self.dev.receive()
        rec_batch, rec_cnt = rec
        self.assertEqual(1, rec_cnt)
        mock_batch = CanMsgBatch(CanMsg(CanFrame(486, [0, 12, 0])))
        self.assertEqual(rec_batch, mock_batch)
        self.assertEqual(DeviceResult.SUCCESS, self.dev.close_channel())

    def test_receive_block(self):
        self.assertEqual(DeviceResult.SUCCESS, self.dev.open_channel())
        batch = create_batch_from_zcanpro_exporting_file("files/test_batch_1_msg.list")
        can_msg = batch.can_msgs[0]
        self.assertEqual(1, self.dev.send_to_myself(can_msg))
        time.sleep(1)
        rec = self.dev.receive_block()
        rec_batch, rec_cnt = rec
        self.assertEqual(1, rec_cnt)
        mock_batch = CanMsgBatch(CanMsg(CanFrame(486, [0, 12, 0])))
        self.assertEqual(rec_batch, mock_batch)
        self.assertEqual(DeviceResult.SUCCESS, self.dev.close_channel())

    def test_is_device_online(self):
        self.assertTrue(self.dev.is_device_online())


if __name__ == '__main__':
    suit = unittest.TestSuite()
    suit.addTest(TestZlgDevice())
    runner = unittest.TextTestRunner()
    runner.run(suit)



import threading
import time
from queue import Queue

from devices.devices import ZlgDevice, DeviceResult
from framework.canfactory import create_batch_from_zcanpro_exporting_file


class SendWorker(threading.Thread):

    def __init__(self, device):
        threading.Thread.__init__(self)
        self.queue = Queue()
        self.device = device
        self.stop_sending_event = threading.Event()

    def _send_one_loop(self, batch):
        to_myself = batch.to_myself
        for can_msg in batch.can_msgs:
            for cnt in range(can_msg.count):
                if to_myself:
                    self.device.send_to_myself(can_msg)
                else:
                    self.device.send(can_msg)
                time.sleep(can_msg.interval / 1000.0)
        time.sleep(batch.loop_interval / 1000.0)

    def _send_infinity(self, batch):
        while True:
            if self.stop_sending_event.is_set():
                break
            self._send_one_loop(batch)

    def _send_count(self, batch):
        for cnt in range(batch.loop_count):
            if self.stop_sending_event.is_set():
                break
            self._send_one_loop(batch)

    def run(self):
        while True:
            batch = self.queue.get()
            if batch is None:
                break
            while True:
                # if self.stop_sending_event.is_set():
                #     break
                if batch.loop_count == -1:
                    self._send_infinity(batch)
                    break
                elif batch.loop_count > 0:
                    self._send_count(batch)
                    break
                else:
                    raise

            self.stop_sending_event.clear()

    def send_batch(self, batch):
        self.queue.put(batch)

    def stop_sending(self):
        self.stop_sending_event.set()

    def kill(self):
        self.queue.put(None)
        self.join()


class ReceiveWorker(threading.Thread):

    def __init__(self, device, on_receive_fun):
        threading.Thread.__init__(self)
        self.device = device
        self.on_receive_fun = on_receive_fun
        self.stop_receiving_event = threading.Event()

    def run(self):
        while True:
            if self.stop_receiving_event.is_set():
                break
            batch, act_num = self.device.receive_block(1000)
            if act_num > 0 and self.on_receive_fun is not None:
                self.on_receive_fun(batch, act_num)

    def stop_receiving(self):
        self.stop_receiving_event.set()
        self.join()


class CanBox:

    def __init__(self, device):
        self.device_open_flag = False
        self.channel_open_flag = False
        self.device = device
        self.send_worker = None
        self.receive_worker = None

    def is_device_online(self):
        return self.device.is_device_online()

    def open(self, on_receive_callback=None):
        """
        open the device
        :return:
        """
        if not self.device_open_flag:
            ret = self.device.open()
            self.send_worker = SendWorker(self.device)
            self.receive_worker = ReceiveWorker(self.device, on_receive_callback)
            self.send_worker.start()
            self.receive_worker.start()
            self.device_open_flag = True
            return ret == DeviceResult.SUCCESS

        return False

    def open_channel(self):
        if not self.channel_open_flag:
            ret = self.device.open_channel()
            self.channel_open_flag = True
            return ret == DeviceResult.SUCCESS

        return False

    def send_batch(self, batch):
        batch.to_myself = False
        self.send_worker.send_batch(batch)

    def send_batch_to_myself(self, batch):
        batch.to_myself = True
        self.send_worker.send_batch(batch)

    def stop_sending(self):
        self.send_worker.stop_sending()

    def stop_receiving(self):
        self.receive_worker.stop_receiving()

    def close_channel(self):
        if self.channel_open_flag:
            self.send_worker.kill()
            ret = self.device.close_channel()
            self.channel_open_flag = False
            return ret == DeviceResult.SUCCESS

        return False

    def close(self):
        """
        close the device
        :return:
        """
        if self.device_open_flag:
            self.stop_sending()
            self.stop_receiving()
            self.close_channel()
            ret = self.device.close()
            self.device_open_flag = False
            return ret == DeviceResult.SUCCESS

        return False


if __name__ == '__main__':
    dev = ZlgDevice()
    cb = CanBox(dev)
    cb.open()
    cb.send_batch(create_batch_from_zcanpro_exporting_file("../test/dbc.list"))
    time.sleep(1)
    cb.stop_sending()
    cb.close()


import time
from abc import abstractmethod

from devices.zlg.zlgcan import ZCAN, ZCAN_CHANNEL_INIT_CONFIG, ZCAN_TYPE_CAN, ZCAN_Transmit_Data, ZCAN_STATUS_ONLINE, \
    INVALID_DEVICE_HANDLE, INVALID_CHANNEL_HANDLE, ZCAN_STATUS_ERR, ZCAN_SEND_TYPE_NORMAL, \
    ZCAN_SEND_TYPE_SEND_TO_MYSELF
from framework.canfactory import create_batch_from_zcanpro_receive


class DeviceResult:
    SUCCESS = 0
    DEVICE_OPEN_FAIL = 1
    DEVICE_CLOSE_FAIL = 2
    CHANNEL_CONFIG_FAIL = 3
    CHANNEL_OPEN_FAIL = 4
    CHANNEL_CLOSE_FAIL = 5
    SEND_FAIL = 6


class Device:

    def __init__(self):
        self.send_success_count = 0
        self.rec_success_count = 0

    @abstractmethod
    def open(self):
        print("Device #open")
        return DeviceResult.SUCCESS

    @abstractmethod
    def is_device_online(self):
        print("Device #is_device_online")
        return True

    @abstractmethod
    def open_channel(self):
        print("Device #open_channel")
        return DeviceResult.SUCCESS

    @abstractmethod
    def send(self, can_msg):
        self.send_success_count += 1
        print("Device #send [count=%s]: %s" % (self.send_success_count, can_msg))
        pass

    @abstractmethod
    def receive(self):
        self.rec_success_count += 1
        print("Device #receive [count=%s]: " % (self.rec_success_count))


    @abstractmethod
    def receive_block(self, timeout=-1):
        self.rec_success_count += 1
        time.sleep(1)
        print("Device #receive [count=%s]: " % (self.rec_success_count))
        return None, 0

    @abstractmethod
    def close_channel(self):
        print("Device #close_channel")
        return DeviceResult.SUCCESS

    @abstractmethod
    def close(self):
        print("Device #close")
        return DeviceResult.SUCCESS


class ZlgDevice(Device):

    def __init__(self, zlgcan_dll):
        self.zcan = ZCAN(zlgcan_dll)
        self.dev_handle = INVALID_DEVICE_HANDLE
        self.chn_handle = INVALID_CHANNEL_HANDLE
        self.send_success_count = 0
        self.send_fail_count = 0
        self.rec_success_count = 0

    def open(self):
        self.dev_handle = self.zcan.OpenDevice(device_type=4, device_index=0, reserved=0)
        if self.dev_handle is INVALID_DEVICE_HANDLE:
            return DeviceResult.DEVICE_OPEN_FAIL

        return DeviceResult.SUCCESS

    def config(self):
        chn_cfg = ZCAN_CHANNEL_INIT_CONFIG()
        chn_cfg.can_type = ZCAN_TYPE_CAN
        chn_cfg.config.can.timing0 = 0
        chn_cfg.config.can.timing1 = 0x1C
        chn_cfg.config.can.acc_code = 0
        chn_cfg.config.can.acc_mask = 0xFFFFFFFF
        return chn_cfg

    def open_channel(self):
        chn_cfg = self.config()
        self.chn_handle = self.zcan.InitCAN(device_handle=self.dev_handle, can_index=0, init_config=chn_cfg)
        if self.chn_handle is INVALID_CHANNEL_HANDLE:
            return DeviceResult.CHANNEL_CONFIG_FAIL

        ret = self.zcan.StartCAN(self.chn_handle)
        if ret is ZCAN_STATUS_ERR:
            return DeviceResult.CHANNEL_OPEN_FAIL

        return DeviceResult.SUCCESS

    def is_device_online(self):
        return self.dev_handle is not None and \
               self.zcan.DeviceOnLine(self.dev_handle) == ZCAN_STATUS_ONLINE

    def send(self, can_msg, transmit_type=ZCAN_SEND_TYPE_NORMAL):
        if self.chn_handle is None:
            return DeviceResult.SEND_FAIL

        msg = ZCAN_Transmit_Data()
        msg.transmit_type = transmit_type
        msg.frame.can_id = can_msg.id
        msg.frame.rtr = 0
        msg.frame.eff = 0
        msg.frame.can_dlc = len(can_msg.data)
        for i in range(msg.frame.can_dlc):
            msg.frame.data[i] = can_msg.data[i]

        send_num = 1
        send_msgs = (ZCAN_Transmit_Data * send_num)()
        send_msgs[0] = msg

        trans_success_count = self.zcan.Transmit(self.chn_handle, send_msgs, send_num)
        return trans_success_count

    def send_to_myself(self, can_msg):
        return self.send(can_msg, transmit_type=ZCAN_SEND_TYPE_SEND_TO_MYSELF)

    def receive(self):
        rec_num = self.zcan.GetReceiveNum(self.chn_handle, ZCAN_TYPE_CAN)
        if rec_num > 0:
            MAX_RCV_NUM = 10
            read_cnt = MAX_RCV_NUM if rec_num >= MAX_RCV_NUM else rec_num
            receive_msgs, act_num = self.zcan.Receive(self.chn_handle, read_cnt)
            batch = create_batch_from_zcanpro_receive(receive_msgs, act_num)
            return batch, act_num
        return None, 0

    def receive_block(self, timeout=-1):
        read_cnt = 10
        receive_msgs, act_num = self.zcan.Receive(self.chn_handle, read_cnt, timeout)
        if act_num <= 0:
            return None, 0

        batch = create_batch_from_zcanpro_receive(receive_msgs, act_num)
        return batch, act_num

    def close_channel(self):
        if self.chn_handle is INVALID_CHANNEL_HANDLE:
            return DeviceResult.CHANNEL_CLOSE_FAIL

        ret = self.zcan.ResetCAN(self.chn_handle)
        if ret is ZCAN_STATUS_ERR:
            return DeviceResult.CHANNEL_CLOSE_FAIL

        return DeviceResult.SUCCESS

    def close(self):
        if self.dev_handle is INVALID_DEVICE_HANDLE:
            return DeviceResult.DEVICE_CLOSE_FAIL

        ret = self.zcan.CloseDevice(self.dev_handle)
        if ret is ZCAN_STATUS_ERR:
            return DeviceResult.DEVICE_CLOSE_FAIL

        return DeviceResult.SUCCESS


# DEVICE = Device()
# ZLG_DEVICE = ZlgDevice("../devices/zlg/zlgcan.dll")


import json

from framework.canmsg import CanMsgBatch, CanMsg


def create_batch_from_zcanpro_exporting_file(path):
    """
    create CanMsgBatch by turn_light.list
    :param path: the absolute path for turn_light.list which is exported from DBC sending module in ZCanPro
    :return: CanMsgBatch instance
    """

    with open(path) as df:
        batch_obj = json.load(df)
        batch = CanMsgBatch()
        batch.loop_count = batch_obj["sendLoopCnt"]
        batch.loop_interval = batch_obj["sendLoopInterval"]
        batch.can_msgs = []
        can_msg_obj_list = batch_obj["msgList"]
        for can_msg_obj in can_msg_obj_list:
            count = can_msg_obj["sendCnt"]
            interval = can_msg_obj["sendInterval"]
            id = can_msg_obj["msgID"]
            data = can_msg_obj["datas"]
            can_msg = CanMsg(id, data, count, interval)
            batch.can_msgs.append(can_msg)

    raise_if_bad_batch(batch)
    return batch


def raise_if_bad_batch(batch):
    if batch is None:
        raise
    can_msgs = batch.can_msgs
    if len(can_msgs) == 0:
        raise

    tmp_id = -1
    for can_msg in can_msgs:
        frame_id = can_msg.id
        if tmp_id == -1:
            tmp_id = frame_id
        elif tmp_id != frame_id:
            raise


def generate_zcanpro_import_file_from_can_msg_batch(batch):
    """
    create turn_light.list which is used by ZCanPro DBC sending module from CanMsgBatch instance
    :param batch: CanMsgBatch
    :return: turn_light.list which is used by ZCanPro DBC sending module
    """
    pass


def create_batch_from_zcanpro_receive(zcanpro_rec_msgs, act_cnt):
    batch = CanMsgBatch()
    batch.can_msgs = []
    for i in range(act_cnt):
        zcanpro_rec_msg = zcanpro_rec_msgs[i]
        zframe = zcanpro_rec_msg.frame
        zcan_id = zframe.can_id
        zcan_dlc = zframe.can_dlc
        zdata = zframe.data[:zcan_dlc]
        can_msg = CanMsg(zcan_id, zdata)
        batch.can_msgs.append(can_msg)

    return batch


def create_batch_from_signal_group(*signal_groups, count=1, loop_count=-1):
    batch = CanMsgBatch()
    for group in signal_groups:
        can_msg = CanMsg(id=group.msg_id, data=group.to_data(), count=count, interval=group.interval)
        batch.add(can_msg)
    batch.loop_count = loop_count
    batch.loop_interval = batch.can_msgs[0].interval
    return batch


def create_can_msg_from_signal_group(signal_group):
    return CanMsg(id=signal_group.msg_id, data=signal_group.to_data(), count=1, interval=signal_group.interval)


if __name__ == '__main__':
    create_batch_from_zcanpro_exporting_file("../test/dbc.list")


class DBC:
    def __init__(self):
        self.nodes = set()
        self.frame_id_to_sender_names = dict()
        self.sender_name_to_frame_ids = dict()
        self.signal_name_to_frame_id = dict()
        self.signal_name_to_choices = dict()
        self.frame_id_to_interval = dict()
        self.frame_id_to_signal_names = dict()
        self.dbs = list()
        pass

    def _set_node_list(self, db):
        for node in db.nodes:
            self.nodes.add(node.name)

    def _set_frame_id_to_sender_names(self, db):
        for msg in db.messages:
            if msg.frame_id not in self.frame_id_to_sender_names.keys():
                self.frame_id_to_sender_names[msg.frame_id] = msg.senders

    def _set_sender_name_to_frame_ids(self, db):
        for msg in db.messages:
            senders = msg.senders
            for sender in senders:
                if self.sender_name_to_frame_ids.get(sender) is None:
                    self.sender_name_to_frame_ids[sender] = set()
                self.sender_name_to_frame_ids[sender].add(msg.frame_id)

    def _set_signal_name_to_frame_id(self, db):
        for msg in db.messages:
            for signal in msg.signals:
                sig_name = signal.name
                if self.signal_name_to_frame_id.get(sig_name) is None:
                    self.signal_name_to_frame_id[sig_name] = msg.frame_id

    def _set_signal_name_to_choices(self, db):
        for msg in db.messages:
            for signal in msg.signals:
                sig_name = signal.name
                choices = signal.choices
                if self.signal_name_to_choices.get(sig_name) is None:
                    self.signal_name_to_choices[sig_name] = choices

    def _set_frame_id_to_interval(self, db):
        for msg in db.messages:
            if self.frame_id_to_interval.get(msg.frame_id) is None:
                self.frame_id_to_interval[msg.frame_id] = msg.cycle_time

    def _set_frame_id_to_signal_names(self, db):
        for msg in db.messages:
            if self.frame_id_to_signal_names.get(msg.frame_id) is None:
                self.frame_id_to_signal_names[msg.frame_id] = set()
            for sig in msg.signals:
                sig_name = sig.name
                self.frame_id_to_signal_names[msg.frame_id].add(sig_name)

    def _create_attr(self, db):
        msgs = db.messages
        for msg in msgs:
            if msg.name not in self.__dict__.keys():
                self.__dict__[msg.name] = msg
            sigs = msg.signals
            for sig in sigs:
                if sig.name not in self.__dict__.keys():
                    self.__dict__[msg.name].__dict__[sig.name] = sig
                chs = sig.choices
                if chs is not None:
                    for val, name in chs.items():
                        name = name.replace(" ", "_")
                        name = name.replace("0", "Zero")
                        self.__dict__[msg.name].__dict__[sig.name].__dict__[name] = val

    def load(self, db):
        self.dbs.append(db)
        self._set_node_list(db)
        self._set_frame_id_to_sender_names(db)
        self._set_sender_name_to_frame_ids(db)
        self._set_signal_name_to_frame_id(db)
        self._set_signal_name_to_choices(db)
        self._set_frame_id_to_interval(db)
        self._set_frame_id_to_signal_names(db)
        self._create_attr(db)

    def check_signal_name(self, signal_name):
        return signal_name in self.signal_name_to_frame_id.keys()

    def find_choices_by_signal_name(self, signal_name):
        return self.signal_name_to_choices.get(signal_name)

    def find_msg_id_by_signal_name(self, signal_name):
        return self.signal_name_to_frame_id.get(signal_name)

    def find_msg_interval_by_msg_id(self, msg_id):
        return self.frame_id_to_interval.get(msg_id)

    def find_signal_names_by_msg_id(self, msg_id):
        return self.frame_id_to_signal_names.get(msg_id)

    def encode_message(self, msg_id, signal_name_to_value):
        ret = b'\x00'
        for db in self.dbs:
            data = db.encode_message(msg_id, signal_name_to_value)
            ret = (int.from_bytes(ret, 'big') | int.from_bytes(data, 'big')).to_bytes(max(len(ret), len(data)), 'big')
        return ret

    def decode_message(self, msg_id, data, decode_choices=False):
        signal_name_to_choice_or_value = {}
        for db in self.dbs:
            data_dict = db.decode_message(msg_id, data, decode_choices)
            signal_name_to_choice_or_value = dict(signal_name_to_choice_or_value, **data_dict)
        return signal_name_to_choice_or_value


# -*- coding:utf-8 -*-
#  zlgcan.py
#
#  ~~~~~~~~~~~~
#
#  ZLGCAN API
#
#  ~~~~~~~~~~~~
#
#  ------------------------------------------------------------------
#  Author : guochuangjian    
#  Last change: 21.02.2019
#
#  Language: Python 2.7, 3.6
#  ------------------------------------------------------------------
#
import platform
from ctypes import *

ZCAN_DEVICE_TYPE = c_uint

INVALID_DEVICE_HANDLE  = 0
INVALID_CHANNEL_HANDLE = 0

'''
 Device Type
'''
ZCAN_PCI5121          = ZCAN_DEVICE_TYPE(1) 
ZCAN_PCI9810          = ZCAN_DEVICE_TYPE(2) 
ZCAN_USBCAN1          = ZCAN_DEVICE_TYPE(3) 
ZCAN_USBCAN2          = ZCAN_DEVICE_TYPE(4) 
ZCAN_PCI9820          = ZCAN_DEVICE_TYPE(5) 
ZCAN_CAN232           = ZCAN_DEVICE_TYPE(6) 
ZCAN_PCI5110          = ZCAN_DEVICE_TYPE(7) 
ZCAN_CANLITE          = ZCAN_DEVICE_TYPE(8) 
ZCAN_ISA9620          = ZCAN_DEVICE_TYPE(9) 
ZCAN_ISA5420          = ZCAN_DEVICE_TYPE(10)
ZCAN_PC104CAN         = ZCAN_DEVICE_TYPE(11)
ZCAN_CANETUDP         = ZCAN_DEVICE_TYPE(12)
ZCAN_CANETE           = ZCAN_DEVICE_TYPE(12)
ZCAN_DNP9810          = ZCAN_DEVICE_TYPE(13)
ZCAN_PCI9840          = ZCAN_DEVICE_TYPE(14)
ZCAN_PC104CAN2        = ZCAN_DEVICE_TYPE(15)
ZCAN_PCI9820I         = ZCAN_DEVICE_TYPE(16)
ZCAN_CANETTCP         = ZCAN_DEVICE_TYPE(17)
ZCAN_PCIE_9220        = ZCAN_DEVICE_TYPE(18)
ZCAN_PCI5010U         = ZCAN_DEVICE_TYPE(19)
ZCAN_USBCAN_E_U       = ZCAN_DEVICE_TYPE(20)
ZCAN_USBCAN_2E_U      = ZCAN_DEVICE_TYPE(21)
ZCAN_PCI5020U         = ZCAN_DEVICE_TYPE(22)
ZCAN_EG20T_CAN        = ZCAN_DEVICE_TYPE(23)
ZCAN_PCIE9221         = ZCAN_DEVICE_TYPE(24)
ZCAN_WIFICAN_TCP      = ZCAN_DEVICE_TYPE(25)
ZCAN_WIFICAN_UDP      = ZCAN_DEVICE_TYPE(26)
ZCAN_PCIe9120         = ZCAN_DEVICE_TYPE(27)
ZCAN_PCIe9110         = ZCAN_DEVICE_TYPE(28)
ZCAN_PCIe9140         = ZCAN_DEVICE_TYPE(29)
ZCAN_USBCAN_4E_U      = ZCAN_DEVICE_TYPE(31)
ZCAN_CANDTU_200UR     = ZCAN_DEVICE_TYPE(32)
ZCAN_CANDTU_MINI      = ZCAN_DEVICE_TYPE(33)
ZCAN_USBCAN_8E_U      = ZCAN_DEVICE_TYPE(34)
ZCAN_CANREPLAY        = ZCAN_DEVICE_TYPE(35)
ZCAN_CANDTU_NET       = ZCAN_DEVICE_TYPE(36)
ZCAN_CANDTU_100UR     = ZCAN_DEVICE_TYPE(37)
ZCAN_PCIE_CANFD_100U  = ZCAN_DEVICE_TYPE(38)
ZCAN_PCIE_CANFD_200U  = ZCAN_DEVICE_TYPE(39)
ZCAN_PCIE_CANFD_400U  = ZCAN_DEVICE_TYPE(40)
ZCAN_USBCANFD_200U    = ZCAN_DEVICE_TYPE(41)
ZCAN_USBCANFD_100U    = ZCAN_DEVICE_TYPE(42)
ZCAN_USBCANFD_MINI    = ZCAN_DEVICE_TYPE(43)
ZCAN_CANFDCOM_100IE   = ZCAN_DEVICE_TYPE(44)
ZCAN_CANSCOPE         = ZCAN_DEVICE_TYPE(45)
ZCAN_CLOUD            = ZCAN_DEVICE_TYPE(46)
ZCAN_CANDTU_NET_400   = ZCAN_DEVICE_TYPE(47)
ZCAN_VIRTUAL_DEVICE   = ZCAN_DEVICE_TYPE(99)

'''
 Interface return status
'''
ZCAN_STATUS_ERR         = 0
ZCAN_STATUS_OK          = 1
ZCAN_STATUS_ONLINE      = 2
ZCAN_STATUS_OFFLINE     = 3
ZCAN_STATUS_UNSUPPORTED = 4

'''
 CAN type
'''
ZCAN_TYPE_CAN    = c_uint(0)
ZCAN_TYPE_CANFD  = c_uint(1)

"""
author: chenhewen
from ZLGCAN接口函数使用手册
desc: 我们将发送类型定义在这里，这个本应该ZLG定义在这里的，但是却没有
0=正常发送，1=单次发送，2=自发自收，3=单次自发自收
"""
ZCAN_SEND_TYPE_NORMAL = 0
ZCAN_SEND_TYPE_NORMAL_ONCE = 1
ZCAN_SEND_TYPE_SEND_TO_MYSELF = 2
ZCAN_SEND_TYPE_SEND_TO_MYSELF_ONCE = 3

'''
 Device information
'''
class ZCAN_DEVICE_INFO(Structure):
    _fields_ = [("hw_Version", c_ushort),
                ("fw_Version", c_ushort),
                ("dr_Version", c_ushort), 
                ("in_Version", c_ushort), 
                ("irq_Num", c_ushort),
                ("can_Num", c_ubyte),
                ("str_Serial_Num", c_ubyte * 20),
                ("str_hw_Type", c_ubyte * 40),
                ("reserved", c_ushort * 4)]

    def __str__(self):
        return "Hardware Version:%s\nFirmware Version:%s\nDriver Interface:%s\nInterface Interface:%s\nInterrupt Number:%d\nCAN Number:%d\nSerial:%s\nHardware Type:%s\n" %( \
                self.hw_version, self.fw_version, self.dr_version, self.in_version, self.irq_num, self.can_num, self.serial, self.hw_type)
                
    def _version(self, version):
        return ("V%02x.%02x" if version // 0xFF >= 9 else "V%d.%02x") % (version // 0xFF, version & 0xFF)
    
    @property
    def hw_version(self):
        return self._version(self.hw_Version)

    @property
    def fw_version(self):
        return self._version(self.fw_Version)
    
    @property
    def dr_version(self):
        return self._version(self.dr_Version)
    
    @property
    def in_version(self):
        return self._version(self.in_Version)

    @property
    def irq_num(self):
        return self.irq_Num

    @property
    def can_num(self):
        return self.can_Num

    @property
    def serial(self):
        serial = ''
        for c in self.str_Serial_Num:
            if c > 0: 
               serial += chr(c)
            else:
                break 
        return serial

    @property
    def hw_type(self):
        hw_type = ''
        for c in self.str_hw_Type:
            if c > 0:
                hw_type += chr(c)
            else:
                break
        return hw_type

class _ZCAN_CHANNEL_CAN_INIT_CONFIG(Structure):
    _fields_ = [("acc_code", c_uint),
                ("acc_mask", c_uint),
                ("reserved", c_uint),
                ("filter",   c_ubyte),
                ("timing0",  c_ubyte),
                ("timing1",  c_ubyte),
                ("mode",     c_ubyte)]

class _ZCAN_CHANNEL_CANFD_INIT_CONFIG(Structure):
    _fields_ = [("acc_code",     c_uint),
                ("acc_mask",     c_uint),
                ("abit_timing",  c_uint),
                ("dbit_timing",  c_uint),
                ("brp",          c_uint),
                ("filter",       c_ubyte),
                ("mode",         c_ubyte),
                ("pad",          c_ushort),
                ("reserved",     c_uint)]

class _ZCAN_CHANNEL_INIT_CONFIG(Union):
    _fields_ = [("can", _ZCAN_CHANNEL_CAN_INIT_CONFIG), ("canfd", _ZCAN_CHANNEL_CANFD_INIT_CONFIG)]

class ZCAN_CHANNEL_INIT_CONFIG(Structure):
    _fields_ = [("can_type", c_uint),
                ("config", _ZCAN_CHANNEL_INIT_CONFIG)]

class ZCAN_CHANNEL_ERR_INFO(Structure):
    _fields_ = [("error_code", c_uint),
                ("passive_ErrData", c_ubyte * 3),
                ("arLost_ErrData", c_ubyte)]

class ZCAN_CHANNEL_STATUS(Structure):
    _fields_ = [("errInterrupt", c_ubyte),
                ("regMode",      c_ubyte),
                ("regStatus",    c_ubyte), 
                ("regALCapture", c_ubyte),
                ("regECCapture", c_ubyte),
                ("regEWLimit",   c_ubyte),
                ("regRECounter", c_ubyte),
                ("regTECounter", c_ubyte),
                ("Reserved",     c_ubyte)]

class ZCAN_CAN_FRAME(Structure):
    _fields_ = [("can_id",  c_uint, 29),
                ("err",     c_uint, 1),
                ("rtr",     c_uint, 1),
                ("eff",     c_uint, 1), 
                ("can_dlc", c_ubyte),
                ("__pad",   c_ubyte),
                ("__res0",  c_ubyte),
                ("__res1",  c_ubyte),
                ("data",    c_ubyte * 8)]

class ZCAN_CANFD_FRAME(Structure):
    _fields_ = [("can_id", c_uint, 29), 
                ("err",    c_uint, 1),
                ("rtr",    c_uint, 1),
                ("eff",    c_uint, 1), 
                ("len",    c_ubyte),
                ("brs",    c_ubyte, 1),
                ("esi",    c_ubyte, 1),
                ("__res",  c_ubyte, 6),
                ("__res0", c_ubyte),
                ("__res1", c_ubyte),
                ("data",   c_ubyte * 64)]

class ZCAN_Transmit_Data(Structure):
    _fields_ = [("frame", ZCAN_CAN_FRAME), ("transmit_type", c_uint)]

class ZCAN_Receive_Data(Structure):
    _fields_  = [("frame", ZCAN_CAN_FRAME), ("timestamp", c_ulonglong)]

class ZCAN_TransmitFD_Data(Structure):
    _fields_ = [("frame", ZCAN_CANFD_FRAME), ("transmit_type", c_uint)]

class ZCAN_ReceiveFD_Data(Structure):
    _fields_ = [("frame", ZCAN_CANFD_FRAME), ("timestamp", c_ulonglong)]

class ZCAN_AUTO_TRANSMIT_OBJ(Structure):
    _fields_ = [("enable",   c_ushort),
                ("index",    c_ushort),
                ("interval", c_uint),
                ("obj",      ZCAN_Transmit_Data)]

class ZCANFD_AUTO_TRANSMIT_OBJ(Structure):
    _fields_ = [("enable",   c_ushort),
                ("index",    c_ushort),
                ("interval", c_uint),
                ("obj",      ZCAN_TransmitFD_Data)]

class IProperty(Structure):
    _fields_ = [("SetValue", c_void_p), 
                ("GetValue", c_void_p),
                ("GetPropertys", c_void_p)]

class ZCAN(object):
    def __init__(self, zlgcan_dll):
        if platform.system() == "Windows":
            self.__dll = windll.LoadLibrary(zlgcan_dll)
        else:
            print("No support now!")
        if self.__dll == None:
            print("DLL couldn't be loaded!")

    def OpenDevice(self, device_type, device_index, reserved):
        try:
            return self.__dll.ZCAN_OpenDevice(device_type, device_index, reserved)
        except:
            print("Exception on OpenDevice!") 
            raise

    def CloseDevice(self, device_handle):
        try:
            return self.__dll.ZCAN_CloseDevice(device_handle)
        except:
            print("Exception on CloseDevice!")
            raise

    def GetDeviceInf(self, device_handle):
        try:
            info = ZCAN_DEVICE_INFO()
            ret = self.__dll.ZCAN_GetDeviceInf(device_handle, byref(info))
            return info if ret == ZCAN_STATUS_OK else None
        except:
            print("Exception on ZCAN_GetDeviceInf")
            raise

    def DeviceOnLine(self, device_handle):
        try:
            return self.__dll.ZCAN_IsDeviceOnLine(device_handle)
        except:
            print("Exception on ZCAN_ZCAN_IsDeviceOnLine!")
            raise

    def InitCAN(self, device_handle, can_index, init_config):
        try:
            return self.__dll.ZCAN_InitCAN(device_handle, can_index, byref(init_config))
        except:
            print("Exception on ZCAN_InitCAN!")
            raise

    def StartCAN(self, chn_handle):
        try:
            return self.__dll.ZCAN_StartCAN(chn_handle)
        except:
            print("Exception on ZCAN_StartCAN!")
            raise

    def ResetCAN(self, chn_handle):
        try:
            return self.__dll.ZCAN_ResetCAN(chn_handle)
        except:
            print("Exception on ZCAN_ResetCAN!")
            raise

    def ClearBuffer(self, chn_handle):
        try:
            return self.__dll.ZCAN_ClearBuffer(chn_handle)
        except:
            print("Exception on ZCAN_ClearBuffer!")
            raise

    def ReadChannelErrInfo(self, chn_handle):
        try:
            ErrInfo = ZCAN_CHANNEL_ERR_INFO()
            ret = self.__dll.ZCAN_ReadChannelErrInfo(chn_handle, byref(ErrInfo))
            return ErrInfo if ret == ZCAN_STATUS_OK else None
        except:
            print("Exception on ZCAN_ReadChannelErrInfo!")
            raise

    def ReadChannelStatus(self, chn_handle):
        try:
            status = ZCAN_CHANNEL_STATUS()
            ret = self.__dll.ZCAN_ReadChannelStatus(chn_handle, byref(status))
            return status if ret == ZCAN_STATUS_OK else None
        except:
            print("Exception on ZCAN_ReadChannelStatus!")
            raise

    def GetReceiveNum(self, chn_handle, can_type = ZCAN_TYPE_CAN):
        try:
            return self.__dll.ZCAN_GetReceiveNum(chn_handle, can_type)
        except:
            print("Exception on ZCAN_GetReceiveNum!")
            raise

    def Transmit(self, chn_handle, std_msg, len):
        try:
            return self.__dll.ZCAN_Transmit(chn_handle, byref(std_msg), len)
        except:
            print("Exception on ZCAN_Transmit!")
            raise

    def Receive(self, chn_handle, rcv_num, wait_time = c_int(-1)):
        try:
            rcv_can_msgs = (ZCAN_Receive_Data * rcv_num)()
            ret = self.__dll.ZCAN_Receive(chn_handle, byref(rcv_can_msgs), rcv_num, wait_time)
            return rcv_can_msgs, ret
        except:
            print("Exception on ZCAN_Receive!")
            raise
    
    def TransmitFD(self, chn_handle, fd_msg, len):
        try:
            return self.__dll.ZCAN_TransmitFD(chn_handle, byref(fd_msg), len)
        except:
            print("Exception on ZCAN_TransmitFD!")
            raise
    
    def ReceiveFD(self, chn_handle, rcv_num, wait_time = c_int(-1)):
        try:
            rcv_canfd_msgs = (ZCAN_ReceiveFD_Data * rcv_num)()
            ret = self.__dll.ZCAN_ReceiveFD(chn_handle, byref(rcv_canfd_msgs), rcv_num, wait_time)
            return rcv_canfd_msgs, ret
        except:
            print("Exception on ZCAN_ReceiveFD!")
            raise

    def GetIProperty(self, device_handle):
        try:
            self.__dll.GetIProperty.restype = POINTER(IProperty)
            return self.__dll.GetIProperty(device_handle)
        except:
            print("Exception on ZCAN_GetIProperty!")
            raise

    def SetValue(self, iproperty, path, value):
        try:
            func = CFUNCTYPE(c_uint, c_char_p, c_char_p)(iproperty.contents.SetValue)
            return func(c_char_p(path.encode("utf-8")), c_char_p(value.encode("utf-8")))
        except:
            print("Exception on IProperty SetValue")
            raise

    def GetValue(self, iproperty, path):
        try:
            func = CFUNCTYPE(c_char_p, c_char_p)(iproperty.contents.GetValue)
            return func(c_char_p(path.encode))
        except:
            print("Exception on IProperty GetValue")
            raise

    def ReleaseIProperty(self, iproperty):
        try:
            return self.__dll.ReleaseIProperty(iproperty)
        except:
            print("Exception on ZCAN_ReleaseIProperty!")
            raise
###############################################################################
'''
USBCANFD-MINI Demo
'''
def can_start(zcanlib, device_handle, chn):
    ip = zcanlib.GetIProperty(device_handle)
    ret = zcanlib.SetValue(ip, str(chn) + "/clock", "60000000")
    if ret != ZCAN_STATUS_OK:
        print("Set CH%d CANFD clock failed!" %(chn))
    ret = zcanlib.SetValue(ip, str(chn) + "/canfd_standard", "0")
    if ret != ZCAN_STATUS_OK:
        print("Set CH%d CANFD standard failed!" %(chn))
    ret = zcanlib.SetValue(ip, str(chn) + "/initenal_resistance", "1")
    if ret != ZCAN_STATUS_OK:
        print("Open CH%d resistance failed!" %(chn))
    zcanlib.ReleaseIProperty(ip) 

    chn_init_cfg = ZCAN_CHANNEL_INIT_CONFIG()
    chn_init_cfg.can_type = ZCAN_TYPE_CANFD
    chn_init_cfg.config.canfd.abit_timing = 101166 #1Mbps
    chn_init_cfg.config.canfd.dbit_timing = 101166 #1Mbps
    chn_init_cfg.config.canfd.mode        = 0
    chn_handle = zcanlib.InitCAN(device_handle, chn, chn_init_cfg)
    if chn_handle is None:
        return None
    zcanlib.StartCAN(chn_handle)
    return chn_handle

if __name__ == "__main__":
    zcanlib = ZCAN() 
    handle = zcanlib.OpenDevice(ZCAN_USBCAN2, 0,0)
    if handle == INVALID_DEVICE_HANDLE:
        print("Open Device failed!")
        exit(0)
    print("device handle:%d." %(handle))

    info = zcanlib.GetDeviceInf(handle)
    print("Device Information:\n%s" %(info))

    #Start CAN
    chn_handle = can_start(zcanlib, handle, 0)
    print("channel handle:%d." %(chn_handle))

    chn_handle_rec = can_start(zcanlib, handle, 1)
    print("rec channel handle:%d." %(chn_handle_rec))

    #Send CAN Messages
    transmit_num = 10
    msgs = (ZCAN_Transmit_Data * transmit_num)()
    for i in range(transmit_num):
        msgs[i].transmit_type = 1
        msgs[i].frame.eff     = 0 #extern frame
        msgs[i].frame.rtr     = 0 #remote frame
        msgs[i].frame.can_id  = i
        msgs[i].frame.can_dlc = 8
        for j in range(msgs[i].frame.can_dlc):
            msgs[i].frame.data[j] = j
    ret = zcanlib.Transmit(chn_handle, msgs, transmit_num)
    print("Tranmit Num: %d." % ret)

    #Receive Messages
    while True:
        rcv_num = zcanlib.GetReceiveNum(chn_handle, ZCAN_TYPE_CAN)
        if rcv_num:
            print("Receive CAN message number:%d" % rcv_num)
            rcv_msg, rcv_num = zcanlib.Receive(chn_handle, rcv_num)
            for i in range(rcv_num):
                print("[%d]:ts:%d, id:%d, dlc:%d, eff:%d, rtr:%d, data:%s" %(i, rcv_msg[i].timestamp, 
                      rcv_msg[i].frame.can_id, rcv_msg[i].frame.can_dlc, 
                      rcv_msg[i].frame.eff, rcv_msg[i].frame.rtr,
                      ''.join(str(rcv_msg[i].frame.data[j]) + ' ' for j in range(rcv_msg[i].frame.can_dlc))))
        else:
            break

    #Close CAN
    zcanlib.ResetCAN(chn_handle)
    #Close Device
    zcanlib.CloseDevice(handle)



from framework.cansignal import Signal, SignalGroup
from framework.concurrentcanbox import SendControl
from framework.trigger import OnReceiveSignalTrigger, OnReceiveAnyMsgTrigger, OnReceiveMsgTrigger, \
    OnReceiveMsgExcludeIdsTrigger, OnKeyPressedTrigger, OnKeyReleaseTrigger, OnReceiveSignalGroupTrigger, \
    OnReceiveMsgIdsTrigger


class VirtualNode:

    def __init__(self, name, dbc):
        self.dbc = dbc
        self.name = name
        self.send_worker = None
        self.msg_triggers = []
        self.key_triggers = []

    def set_send_worker(self, send_worker):
        self.send_worker = send_worker

    def _check_send_id(self, frame_id):
        return frame_id in self.id_set()

    def _check_send_batch(self, batch):
        for can_msg in batch.can_msgs:
            if not self._check_send_id(can_msg.id):
                return False
        return True

    def id_set(self):
        return self.dbc.sender_name_to_frame_ids.get(self.name)

    def send(self, batch):
        if not self._check_send_batch(batch):
            raise
        control = SendControl(send_from=self.name)
        return self.send_worker.send(batch, control)

    def change(self, signal, value):
        pass

    def receive_msg(self, can_msg):
        for trigger in self.msg_triggers:
            if trigger.check(can_msg):
                trigger.do_it(can_msg)

    def on_receive_any_msg(self):
        trigger = OnReceiveAnyMsgTrigger(self)
        self.msg_triggers.append(trigger)
        return trigger

    def on_receive_msg_exclude_ids(self, *msg_ids):
        trigger = OnReceiveMsgExcludeIdsTrigger(self, msg_ids)
        self.msg_triggers.append(trigger)
        return trigger

    def on_receive_msg_ids(self, *msg_ids):
        trigger = OnReceiveMsgIdsTrigger(self, msg_ids)
        self.msg_triggers.append(trigger)
        return trigger

    def on_receive_msg(self, can_msg):
        trigger = OnReceiveMsgTrigger(self, can_msg)
        self.msg_triggers.append(trigger)
        return trigger

    def on_receive_signal(self, name, value):
        sig = Signal(self.dbc, name, value)
        trigger = OnReceiveSignalTrigger(self, sig)
        self.msg_triggers.append(trigger)
        return trigger

    def on_receive_signals(self, name_value_dict):
        sig_group = SignalGroup(self.dbc)
        for name, value in name_value_dict.items():
            sig_group.update_signal_value(name, value)
        trigger = OnReceiveSignalGroupTrigger(self, sig_group)
        self.msg_triggers.append(trigger)
        return trigger

    def receive_key_event(self, key_event):
        for trigger in self.key_triggers:
            if trigger.check(key_event):
                trigger.do_it(key_event)

    def on_key_pressed(self, key_name):
        trigger = OnKeyPressedTrigger(self, key_name)
        self.key_triggers.append(trigger)
        return trigger

    def on_key_released(self, key_name):
        trigger = OnKeyReleaseTrigger(self, key_name)
        self.key_triggers.append(trigger)
        return trigger



import threading
import time
from concurrent.futures import wait
from concurrent.futures.thread import ThreadPoolExecutor

import keyboard

from devices.devices import DeviceResult


class SendControl:

    def __init__(self, send_from=None, stop_sending_event=None):
        if stop_sending_event is None:
            stop_sending_event = threading.Event()
        self.stop_sending_event = stop_sending_event
        self.send_from = send_from


class SendHandle:

    msg_id_to_handle = dict()

    def __init__(self, batch, future):
        self.batch = batch
        self.future = future
        self.stop_sending_event = batch.control.stop_sending_event

    def stop_sending(self):
        self.future.cancel()
        self.stop_sending_event.set()
        SendHandle.remove_handle_on_flight(self)

    def after(self, sec=None):
        wait([self.future], sec)
        return self

    def stop_after(self, sec=None):
        wait([self.future], sec)
        self.stop_sending()

    def wait(self, sec=None):
        wait([self.future], timeout=sec)

    @staticmethod
    def add_handle_on_flight(handle):
        msg_id = handle.batch.can_msgs[0].id
        SendHandle.msg_id_to_handle[msg_id] = handle

    @staticmethod
    def remove_handle_on_flight(handle):
        msg_id = handle.batch.can_msgs[0].id
        SendHandle.msg_id_to_handle.pop(msg_id)

    @staticmethod
    def get_msg_id_on_flight(msg_id):
        return SendHandle.msg_id_to_handle.get(msg_id)


class BatchSendWorker:

    def __init__(self, device, bus):
        self.executor = ThreadPoolExecutor(max_workers=11)
        self.dispatch_future = None
        self.device = device
        self.bus = bus

    def _send_one_loop(self, batch):
        for can_msg in batch.can_msgs:
            # down the control to the msg
            can_msg.control = batch.control
            for cnt in range(can_msg.count):
                self.device.send(can_msg)
                self.bus.broadcast_msg(can_msg)
                time.sleep(can_msg.interval / 1000.0)
        time.sleep(batch.loop_interval / 1000.0)

    def _send_infinity(self, batch):
        # print("send_infinity: ", batch)
        while True:
            if batch.control.stop_sending_event.is_set():
                # print("stop_sending_infinity: ", batch)
                break
            self._send_one_loop(batch)

    def _send_count(self, batch):
        # print("send_count: ", batch)
        for cnt in range(batch.loop_count):
            if batch.control.stop_sending_event.is_set():
                # print("stop_sending_count: ", batch)
                break
            self._send_one_loop(batch)

    def _send_batch(self, batch):
        if batch.loop_count == -1:
            self._send_infinity(batch)
        elif batch.loop_count > 0:
            self._send_count(batch)

    def send(self, batch, control=None):
        if control is None:
            control = SendControl()
        batch.control = control
        handle = SendHandle.get_msg_id_on_flight(batch.can_msgs[0].id)
        if handle is not None:
            handle.stop_sending()

        future = self.executor.submit(self._send_batch, batch)
        handle = SendHandle(batch, future)
        SendHandle.add_handle_on_flight(handle)
        return handle

    def kill(self):
        self.executor.shutdown(wait=False)


class BatchReceiveWorker:

    def __init__(self, device, bus):
        self.device = device
        self.bus = bus
        self.stop_receiving_event = threading.Event()
        self.receive_executor = ThreadPoolExecutor(1)

    def _receive(self):
        while True:
            batch, cnt = self.device.receive_block(1)
            if self.stop_receiving_event.is_set():
                break
            elif cnt > 0:
                batch.control = SendControl()
                for can_msg in batch.can_msgs:
                    can_msg.control = batch.control
                    self.bus.broadcast_msg(can_msg)

    def receive(self):
        self.receive_executor.submit(self._receive)

    def stop_receiving(self):
        self.stop_receiving_event.set()


class KeyWorker:

    def __init__(self, bus):
        self.bus = bus
        self.stop_listening_event = threading.Event()
        self.receive_executor = ThreadPoolExecutor(1)

    def _listen(self):
        while True:
            # TODO: bug! if bock, no way to run into here
            if self.stop_listening_event.is_set():
                break
            event = keyboard.read_event(suppress=False)
            self.bus.broadcast_key_event(event)

    def start_listening(self):
        self.receive_executor.submit(self._listen)

    def stop_listening(self):
        self.stop_listening_event.set()


class ConcurrentCanBox:

    def __init__(self, device):
        self.device = device
        self.device_open_flag = False
        self.channel_open_flag = False
        self.send_worker = BatchSendWorker(device)

    def open(self):
        if not self.device_open_flag:
            ret = self.device.open()
            self.send_worker = BatchSendWorker(self.device)
            self.device_open_flag = True
            return ret == DeviceResult.SUCCESS

    def send(self, batch):
        return self.send_worker.send(batch)

    def open_channel(self):
        if not self.channel_open_flag:
            ret = self.device.open_channel()
            self.channel_open_flag = True
            return ret == DeviceResult.SUCCESS

        return False

    def close_channel(self):
        if self.channel_open_flag:
            ret = self.device.close_channel()
            self.channel_open_flag = False
            return ret == DeviceResult.SUCCESS

        return False

    def close(self):
        if self.device_open_flag:
            self.close_channel()
            self.send_worker.kill()
            ret = self.device.close()
            self.device_open_flag = False
            return ret == DeviceResult.SUCCESS



#  zlgcan_demo.py
#
#  ~~~~~~~~~~~~
#
#  ZLGCAN USBCANFD Demo
#
#  ~~~~~~~~~~~~
#
#  ------------------------------------------------------------------
#  Author : guochuangjian    
#  Last change: 17.01.2019
#
#  Language: Python 3.6
#  ------------------------------------------------------------------
#
import json
import threading
import time
import tkinter as tk
from tkinter import messagebox
from tkinter import ttk

GRPBOX_WIDTH    = 200
MSGCNT_WIDTH    = 50
MSGID_WIDTH     = 80
MSGDIR_WIDTH    = 60
MSGINFO_WIDTH   = 100
MSGLEN_WIDTH    = 60
MSGDATA_WIDTH   = 200
MSGVIEW_WIDTH   = MSGCNT_WIDTH + MSGID_WIDTH + MSGDIR_WIDTH + MSGINFO_WIDTH + MSGLEN_WIDTH + MSGDATA_WIDTH
MSGVIEW_HEIGHT  = 500
SENDVIEW_HEIGHT = 125

WIDGHT_WIDTH    = GRPBOX_WIDTH + MSGVIEW_WIDTH + 40
WIDGHT_HEIGHT   = MSGVIEW_HEIGHT + SENDVIEW_HEIGHT + 20

MAX_DISPLAY     = 1000
MAX_RCV_NUM     = 10

USBCANFD_TYPE    = (41, 42, 43)
USBCAN_XE_U_TYPE = (20, 21, 31)
USBCAN_I_II_TYPE = (3, 4)
###############################################################################
class PeriodSendThread(object):
    def __init__(self, period_func, args=[], kwargs={}):
        self._thread       = threading.Thread(target=self._run)
        self._function     = period_func
        self._args         = args
        self._kwargs       = kwargs
        self._period       = 0
        self._event        = threading.Event()
        self._period_event = threading.Event() 
        self._terminated   = False 
    
    def start(self):
        self._thread.start()

    def stop(self):
        self._terminated = True
        self._event.set()
        self._thread.join()

    def send_start(self, period):
        self._period = period
        self._event.set()

    def send_stop(self):
        self._period_event.set()

    def _run(self):
        while True:
            self._event.wait()
            self._event.clear()
            if self._terminated:
                break
            self._function(*self._args, **self._kwargs) 
            while not self._period_event.wait(self._period):
                self._function(*self._args, **self._kwargs)
            self._period_event.clear()
###############################################################################
class ZCAN_Demo(tk.Tk):
    def __init__(self):
        super().__init__()
        self.title("ZLGCAN Demo for Python")
        self.resizable(False, False)
        self.geometry(str(WIDGHT_WIDTH) + "x" + str(WIDGHT_HEIGHT) + '+200+100')
        self.protocol("WM_DELETE_WINDOW", self.Form_OnClosing)

        self.DeviceInit()
        self.WidgetsInit()

        self._dev_info = None
        with open("dev_info.json", "r") as fd:
            self._dev_info = json.load(fd)
        if self._dev_info == None:
            print("device info no exist!")
            return 

        self.DeviceInfoInit()
        self.ChnInfoUpdate(self._isOpen)

    def DeviceInit(self):
        self._zcan       = ZCAN() 
        self._dev_handle = INVALID_DEVICE_HANDLE 
        self._can_handle = INVALID_CHANNEL_HANDLE 

        self._isOpen = False
        self._isChnOpen = False

        #current device info
        self._is_canfd = False
        self._res_support = False

        #Transmit and receive count display
        self._tx_cnt = 0
        self._rx_cnt = 0
        self._view_cnt = 0

        #read can/canfd message thread
        self._read_thread = None
        self._terminated = False
        self._lock = threading.RLock()

        #period send var
        self._is_sending   = False
        self._id_increase  = False 
        self._send_num     = 1
        self._send_cnt     = 1
        self._is_canfd_msg = False
        self._send_msgs    = None
        self._send_thread  = None

    def WidgetsInit(self):
        self._dev_frame = tk.Frame(self)
        self._dev_frame.grid(row=0, column=0, padx=2, pady=2, sticky=tk.NSEW)

        # Device connect group
        self.gbDevConnect = tk.LabelFrame(self._dev_frame, height=100, width=GRPBOX_WIDTH, text="设备选择")
        self.gbDevConnect.grid_propagate(0)
        self.gbDevConnect.grid(row=0, column=0, padx=2, pady=2, sticky=tk.NE)
        self.DevConnectWidgetsInit()

        self.gbCANCfg = tk.LabelFrame(self._dev_frame, height=170, width=GRPBOX_WIDTH, text="通道配置")
        self.gbCANCfg.grid(row=1, column=0, padx=2, pady=2, sticky=tk.NSEW)
        self.gbCANCfg.grid_propagate(0)
        self.CANChnWidgetsInit()

        self.gbDevInfo = tk.LabelFrame(self._dev_frame, height=230, width=GRPBOX_WIDTH, text="设备信息")
        self.gbDevInfo.grid(row=2, column=0, padx=2, pady=2, sticky=tk.NSEW)
        self.gbDevInfo.grid_propagate(0)
        self.DevInfoWidgetsInit()

        self.gbMsgDisplay = tk.LabelFrame(height=MSGVIEW_HEIGHT, width=MSGVIEW_WIDTH + 12, text="报文显示")
        self.gbMsgDisplay.grid(row=0, column=1, padx=2, pady=2, sticky=tk.NSEW)
        self.gbMsgDisplay.grid_propagate(0)
        self.MsgDisplayWidgetsInit()

        self.gbMsgSend = tk.LabelFrame(heigh=SENDVIEW_HEIGHT, width=MSGVIEW_WIDTH + 12, text="报文发送")
        self.gbMsgSend.grid(row=2, column=1, padx=2, pady=2, sticky=tk.NSEW)
        self.gbMsgSend.grid_propagate(0)
        self.MsgSendWidgetsInit()

    def DeviceInfoInit(self):
        self.cmbDevType["value"] = tuple([dev_name for dev_name in self._dev_info])
        self.cmbDevType.current(0)

    def DevConnectWidgetsInit(self):
        #Device Type
        tk.Label(self.gbDevConnect, text="设备类型:").grid(row=0, column=0, sticky=tk.E)
        self.cmbDevType = ttk.Combobox(self.gbDevConnect, width=16, state="readonly")
        self.cmbDevType.grid(row=0, column=1, sticky=tk.E)

        #Device Index
        tk.Label(self.gbDevConnect, text="设备索引:").grid(row=1, column=0, sticky=tk.E)
        self.cmbDevIdx = ttk.Combobox(self.gbDevConnect, width=16, state="readonly")
        self.cmbDevIdx.grid(row=1, column=1, sticky=tk.E)
        self.cmbDevIdx["value"] = tuple([i for i in range(4)])
        self.cmbDevIdx.current(0)

        #Open/Close Device
        self.strvDevCtrl = tk.StringVar()
        self.strvDevCtrl.set("打开")
        self.btnDevCtrl = ttk.Button(self.gbDevConnect, textvariable=self.strvDevCtrl, command=self.BtnOpenDev_Click)
        self.btnDevCtrl.grid(row=2, column=0, columnspan=2, pady=2) 

    def CANChnWidgetsInit(self):
        #CAN Channel
        tk.Label(self.gbCANCfg, anchor=tk.W, text="CAN通道:").grid(row=0, column=0, sticky=tk.W)
        self.cmbCANChn = ttk.Combobox(self.gbCANCfg, width=12, state="readonly")
        self.cmbCANChn.grid(row=0, column=1, sticky=tk.E)

        #Work Mode
        tk.Label(self.gbCANCfg, anchor=tk.W, text="工作模式:").grid(row=1, column=0, sticky=tk.W)
        self.cmbCANMode = ttk.Combobox(self.gbCANCfg, width=12, state="readonly")
        self.cmbCANMode.grid(row=1, column=1, sticky=tk.E)

        #CAN Baudrate 
        tk.Label(self.gbCANCfg, anchor=tk.W, text="波特率:").grid(row=2, column=0, sticky=tk.W)
        self.cmbBaudrate = ttk.Combobox(self.gbCANCfg, width=12, state="readonly")
        self.cmbBaudrate.grid(row=2, column=1, sticky=tk.W)
        
        #CAN Data Baudrate 
        tk.Label(self.gbCANCfg, anchor=tk.W, text="数据域波特率:").grid(row=3, column=0, sticky=tk.W)
        self.cmbDataBaudrate = ttk.Combobox(self.gbCANCfg, width=12, state="readonly")
        self.cmbDataBaudrate.grid(row=3, column=1, sticky=tk.W)

        #resistance enable
        tk.Label(self.gbCANCfg, anchor=tk.W, text="终端电阻:").grid(row=4, column=0, sticky=tk.W)
        self.cmbResEnable = ttk.Combobox(self.gbCANCfg, width=12, state="readonly")
        self.cmbResEnable.grid(row=4, column=1, sticky=tk.W)

        #CAN Control
        self.strvCANCtrl = tk.StringVar()
        self.strvCANCtrl.set("打开")
        self.btnCANCtrl = ttk.Button(self.gbCANCfg, textvariable=self.strvCANCtrl, command=self.BtnOpenCAN_Click) 
        self.btnCANCtrl.grid(row=5, column=0, columnspan=2, padx=2, pady=2)

    def DevInfoWidgetsInit(self):
        #Hardware Version
        tk.Label(self.gbDevInfo, anchor=tk.W, text="硬件版本:").grid(row=0, column=0, sticky=tk.W)
        self.strvHwVer = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvHwVer).grid(row=0, column=1, sticky=tk.W)

        #Firmware Version
        tk.Label(self.gbDevInfo, anchor=tk.W, text="固件版本:").grid(row=1, column=0, sticky=tk.W)
        self.strvFwVer = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvFwVer).grid(row=1, column=1, sticky=tk.W)

        #Driver Version
        tk.Label(self.gbDevInfo, anchor=tk.W, text="驱动版本:").grid(row=2, column=0, sticky=tk.W)
        self.strvDrVer = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvDrVer).grid(row=2, column=1, sticky=tk.W)

        #Interface Version
        tk.Label(self.gbDevInfo, anchor=tk.W, text="动态库版本:").grid(row=3, column=0, sticky=tk.W)
        self.strvInVer = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvInVer).grid(row=3, column=1, sticky=tk.W)

        #CAN num
        tk.Label(self.gbDevInfo, anchor=tk.W, text="CAN通道数:").grid(row=4, column=0, sticky=tk.W)
        self.strvCANNum = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvCANNum).grid(row=4, column=1, sticky=tk.W)

        #Device Serial
        tk.Label(self.gbDevInfo, anchor=tk.W, text="设备序列号:").grid(row=5, column=0, sticky=tk.W)
        self.strvSerial = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvSerial).grid(row=6, column=0, columnspan=2, sticky=tk.W)
    
        #Hardware type
        tk.Label(self.gbDevInfo, anchor=tk.W, text="硬件类型:").grid(row=7, column=0, sticky=tk.W)
        self.strvHwType = tk.StringVar(value='')
        tk.Label(self.gbDevInfo, anchor=tk.W, textvariable=self.strvHwType).grid(row=8, column=0, columnspan=2, sticky=tk.W)
    
    def MsgDisplayWidgetsInit(self):
        self._msg_frame = tk.Frame(self.gbMsgDisplay, height=MSGVIEW_HEIGHT, width=WIDGHT_WIDTH-GRPBOX_WIDTH+10)
        self._msg_frame.pack(side=tk.TOP)
        
        self.treeMsg = ttk.Treeview(self._msg_frame, height=20, show="headings")
        self.treeMsg["columns"] = ("cnt", "id", "direction", "info", "len", "data")

        self.treeMsg.column("cnt",       anchor = tk.CENTER, width=MSGCNT_WIDTH)
        self.treeMsg.column("id",        anchor = tk.CENTER, width=MSGID_WIDTH)
        self.treeMsg.column("direction", anchor = tk.CENTER, width=MSGDIR_WIDTH)
        self.treeMsg.column("info",      anchor = tk.CENTER, width=MSGINFO_WIDTH)
        self.treeMsg.column("len",       anchor = tk.CENTER, width=MSGLEN_WIDTH)
        self.treeMsg.column("data", width=MSGDATA_WIDTH)

        self.treeMsg.heading("cnt", text="序号")
        self.treeMsg.heading("id", text="帧ID")
        self.treeMsg.heading("direction", text="方向")
        self.treeMsg.heading("info", text="帧信息")
        self.treeMsg.heading("len", text="长度")
        self.treeMsg.heading("data", text="数据")
        
        self.hbar = ttk.Scrollbar(self._msg_frame, orient=tk.HORIZONTAL, command=self.treeMsg.xview)
        self.hbar.pack(side=tk.BOTTOM, fill=tk.X)
        self.vbar = ttk.Scrollbar(self._msg_frame, orient=tk.VERTICAL, command=self.treeMsg.yview)
        self.vbar.pack(side=tk.RIGHT, fill=tk.Y)
        self.treeMsg.configure(xscrollcommand=self.hbar.set, yscrollcommand=self.vbar.set)
        
        self.treeMsg.pack(side=tk.LEFT)
        self.treeMsg.selection_set()
        self.btnClrCnt = ttk.Button(self.gbMsgDisplay, width=10, text="清空", command=self.BtnClrCnt_Click) 
        self.btnClrCnt.pack(side=tk.RIGHT)

        self.strvRxCnt = tk.StringVar()
        self.strvRxCnt.set("0")
        tk.Label(self.gbMsgDisplay, anchor=tk.W, width=5, textvariable=self.strvRxCnt).pack(side=tk.RIGHT)
        tk.Label(self.gbMsgDisplay, width=10, text="接收帧数:").pack(sid=tk.RIGHT)

        self.strvTxCnt = tk.StringVar()
        self.strvTxCnt.set("0")
        tk.Label(self.gbMsgDisplay, anchor=tk.W, width=5, textvariable=self.strvTxCnt).pack(side=tk.RIGHT)
        tk.Label(self.gbMsgDisplay, width=10, text="发送帧数:").pack(side=tk.RIGHT)

    def MsgSendWidgetsInit(self):
        #Send Type
        tk.Label(self.gbMsgSend, anchor=tk.W, text="发送方式:").grid(row=0, column=0, sticky=tk.W)
        self.cmbSendType = ttk.Combobox(self.gbMsgSend, width=8, state="readonly")
        self.cmbSendType.grid(row = 0, column=1, sticky=tk.W) 
        self.cmbSendType["value"] = ("正常发送", "单次发送", "自发自收")
        self.cmbSendType.current(0)

        #CAN Type
        tk.Label(self.gbMsgSend, anchor=tk.W, text="帧类型:").grid(row=0, column=2, sticky=tk.W)
        self.cmbMsgType = ttk.Combobox(self.gbMsgSend, width=6, state="readonly")
        self.cmbMsgType.grid(row = 0, column=3, sticky=tk.W) 
        self.cmbMsgType["value"] = ("标准帧", "扩展帧")
        self.cmbMsgType.current(0)

        #CAN Format 
        tk.Label(self.gbMsgSend, anchor=tk.W, text="帧格式:").grid(row=0, column=4, sticky=tk.W)
        self.cmbMsgFormat = ttk.Combobox(self.gbMsgSend, width=6, state="readonly") 
        self.cmbMsgFormat.grid(row = 0, column=5, sticky=tk.W) 
        self.cmbMsgFormat["value"] = ("数据帧", "远程帧")
        self.cmbMsgFormat.bind("<<ComboboxSelected>>", self.CmbMsgFormatUpdate)
        self.cmbMsgFormat.current(0)

        #CANFD 
        tk.Label(self.gbMsgSend, anchor=tk.W, text="CAN类型:").grid(row=0, column=6, sticky=tk.W)
        self.cmbMsgCANFD = ttk.Combobox(self.gbMsgSend, width=10, state="readonly")
        self.cmbMsgCANFD.grid(row=0, column=7, sticky=tk.W) 
        self.cmbMsgCANFD["value"] = ("CAN", "CANFD", "CANFD BRS")
        self.cmbMsgCANFD.bind("<<ComboboxSelected>>", self.CmbMsgCANFDUpdate)
        self.cmbMsgCANFD.current(0)

        #CAN ID
        tk.Label(self.gbMsgSend, anchor=tk.W, text="帧ID(hex):").grid(row=1, column=0, sticky=tk.W)
        self.entryMsgID = tk.Entry(self.gbMsgSend, width=10, text="100")
        self.entryMsgID.grid(row=1, column=1, sticky=tk.W) 
        self.entryMsgID.insert(0, "100")

        #CAN Length 
        tk.Label(self.gbMsgSend, anchor=tk.W, text="长度:").grid(row=1, column=2, sticky=tk.W)
        self.cmbMsgLen = ttk.Combobox(self.gbMsgSend, width=6, state="readonly")
        self.cmbMsgLen["value"] = tuple([i for i in range(9)])
        self.cmbMsgLen.current(8) 
        self.cmbMsgLen.grid(row=1, column=3, sticky=tk.W) 

        #Data
        tk.Label(self.gbMsgSend, anchor=tk.W, text="数据(hex):").grid(row=1, column=4, sticky=tk.W)
        self.entryMsgData = tk.Entry(self.gbMsgSend, width=30)
        self.entryMsgData.grid(row = 1, column=5, columnspan=4, sticky=tk.W) 
        self.entryMsgData.insert(0, "00 01 02 03 04 05 06 07")

        #send frame number
        tk.Label(self.gbMsgSend, anchor=tk.W, text="发送帧数:").grid(row=2, column=0, sticky=tk.W)
        self.entryMsgNum = tk.Entry(self.gbMsgSend, width=10)
        self.entryMsgNum.grid(row=2, column=1, sticky=tk.W) 
        self.entryMsgNum.insert(0, "1")

        #send frame cnt 
        tk.Label(self.gbMsgSend, anchor=tk.W, text="发送次数:").grid(row=2, column=2, sticky=tk.W)
        self.entryMsgCnt = tk.Entry(self.gbMsgSend, width=8)
        self.entryMsgCnt.grid(row=2, column=3, sticky=tk.W) 
        self.entryMsgCnt.insert(0, "1")

        #send frame period
        tk.Label(self.gbMsgSend, anchor=tk.W, text="发送间隔(ms):").grid(row=2, column=4, sticky=tk.W)
        self.entryMsgPeriod = tk.Entry(self.gbMsgSend, width=8)
        self.entryMsgPeriod.grid(row=2, column=5, sticky=tk.W) 
        self.entryMsgPeriod.insert(0, "0")

        #msg id add
        self.varIDInc = tk.IntVar()
        self.chkbtnIDInc = tk.Checkbutton(self.gbMsgSend, text="ID递增", variable=self.varIDInc)
        self.chkbtnIDInc.grid(row=2, column=6, columnspan=2, sticky=tk.W)

        #Send Butten
        self.strvSend = tk.StringVar()
        self.strvSend.set("发送")
        self.btnMsgSend = ttk.Button(self.gbMsgSend, textvariable=self.strvSend, command=self.BtnSendMsg_Click) 
        self.btnMsgSend.grid(row=3, column=7, padx=2, pady=2)
        self.btnMsgSend["state"] = tk.DISABLED

###############################################################################
### Function 
###############################################################################
    def __dlc2len(self, dlc):
        if dlc <= 8:
            return dlc
        elif dlc == 9:
            return 12
        elif dlc == 10:
            return 16
        elif dlc == 11:
            return 20
        elif dlc == 12:
            return 24
        elif dlc == 13:
            return 32
        elif dlc == 14:
            return 48
        else:
            return 64

    def CANMsg2View(self, msg, is_transmit=True):
        view = []
        view.append(str(self._view_cnt))
        self._view_cnt += 1 
        view.append(hex(msg.can_id)[2:])
        view.append("发送" if is_transmit else "接收")

        str_info = ''
        str_info += 'EXT' if msg.eff else 'STD'
        if msg.rtr:
            str_info += ' RTR'
        view.append(str_info)
        view.append(str(msg.can_dlc))
        if msg.rtr:
            view.append('')
        else:
            view.append(''.join(hex(msg.data[i])[2:] + ' ' for i in range(msg.can_dlc)))
        return view

    def CANFDMsg2View(self, msg, is_transmit=True):
        view = [] 
        view.append(str(self._view_cnt))
        self._view_cnt += 1 
        
        view.append(hex(msg.can_id)[2:])
        view.append("发送" if is_transmit else "接收")

        str_info = ''
        str_info += 'EXT' if msg.eff else 'STD'
        if msg.rtr:
            str_info += ' RTR'
        else:
            str_info += ' FD'
            if msg.brs:
                str_info += ' BRS'
            if msg.esi:
                str_info += ' ESI' 
        view.append(str_info)
        view.append(str(msg.len))
        if msg.rtr:
            view.append('')
        else:
            view.append(''.join(hex(msg.data[i])[2:] + ' ' for i in range(msg.len)))
        return view

    def ChnInfoUpdate(self, is_open):
        #通道信息获取
        cur_dev_info = self._dev_info[self.cmbDevType.get()]
        cur_chn_info = cur_dev_info["chn_info"]
        
        if is_open:
            # 通道 
            self.cmbCANChn["value"] = tuple([i for i in range(cur_dev_info["chn_num"])])
            self.cmbCANChn.current(0)

            # 工作模式
            self.cmbCANMode["value"] = ("正常模式", "只听模式")
            self.cmbCANMode.current(0)

            # 波特率
            self.cmbBaudrate["value"] = tuple([brt for brt in cur_chn_info["baudrate"].keys()])
            self.cmbBaudrate.current(len(self.cmbBaudrate["value"]) - 1)

            if cur_chn_info["is_canfd"] == True:
                # 数据域波特率 
                self.cmbDataBaudrate["value"] = tuple([brt for brt in cur_chn_info["data_baudrate"].keys()])
                self.cmbDataBaudrate.current(0)
                self.cmbDataBaudrate["state"] = "readonly"

            if cur_chn_info["sf_res"] == True:
                self.cmbResEnable["value"] = ("使能", "失能")
                self.cmbResEnable.current(0)
                self.cmbResEnable["state"] = "readonly"

            self.btnCANCtrl["state"] = tk.NORMAL
        else:
            self.cmbCANChn["state"] = tk.DISABLED
            self.cmbCANMode["state"] = tk.DISABLED
            self.cmbBaudrate["state"] = tk.DISABLED
            self.cmbDataBaudrate["state"] = tk.DISABLED
            self.cmbResEnable["state"] = tk.DISABLED
            
            self.cmbCANChn["value"] = ()
            self.cmbCANMode["value"] = ()
            self.cmbBaudrate["value"] = ()
            self.cmbDataBaudrate["value"] = ()
            self.cmbResEnable["value"] = ()

            self.btnCANCtrl["state"] = tk.DISABLED

    def ChnInfoDisplay(self, enable):
        if enable:
            self.cmbCANChn["state"] = "readonly"
            self.cmbCANMode["state"] = "readonly"
            self.cmbBaudrate["state"] = "readonly" 
            if self._is_canfd: 
                self.cmbDataBaudrate["state"] = "readonly" 
            if self._res_support: 
                self.cmbResEnable["state"] = "readonly"
        else:
            self.cmbCANChn["state"] = tk.DISABLED
            self.cmbCANMode["state"] = tk.DISABLED
            self.cmbBaudrate["state"] = tk.DISABLED
            self.cmbDataBaudrate["state"] = tk.DISABLED
            self.cmbResEnable["state"] = tk.DISABLED
    
    def MsgReadThreadFunc(self):
        try:
            while not self._terminated:
                can_num = self._zcan.GetReceiveNum(self._can_handle, ZCAN_TYPE_CAN)
                canfd_num = self._zcan.GetReceiveNum(self._can_handle, ZCAN_TYPE_CANFD)
                if not can_num and not canfd_num:
                    time.sleep(0.005) #wait 5ms  
                    continue

                if can_num:
                    while can_num and not self._terminated:
                        read_cnt = MAX_RCV_NUM if can_num >= MAX_RCV_NUM else can_num
                        # CAUTION: MAX_RCV_NUM is the wrong arguments, it should be replaced with timeout
                        can_msgs, act_num = self._zcan.Receive(self._can_handle, read_cnt, MAX_RCV_NUM)
                        if act_num: 
                            #update data
                            self._rx_cnt += act_num 
                            self.strvRxCnt.set(str(self._rx_cnt))
                            self.ViewDataUpdate(can_msgs, act_num, False, False)
                        else:
                            break
                        can_num -= act_num
                if canfd_num:
                    while canfd_num and not self._terminated:
                        read_cnt = MAX_RCV_NUM if canfd_num >= MAX_RCV_NUM else canfd_num
                        canfd_msgs, act_num = self._zcan.ReceiveFD(self._can_handle, read_cnt, MAX_RCV_NUM)
                        if act_num: 
                            #update data
                            self._rx_cnt += act_num 
                            self.strvRxCnt.set(str(self._rx_cnt))
                            self.ViewDataUpdate(canfd_msgs, act_num, True, False)
                        else:
                            break
                        canfd_num -= act_num
        except:
            print("Error occurred while read CAN(FD) data!")

    def ViewDataUpdate(self, msgs, msgs_num, is_canfd=False, is_send=True):
        with self._lock:
            if is_canfd:
                for i in range(msgs_num):
                    if len(self.treeMsg.get_children()) == MAX_DISPLAY:
                        self.treeMsg.delete(self.treeMsg.get_children()[0])
                    self.treeMsg.insert('', 'end', values=self.CANFDMsg2View(msgs[i].frame, is_send))
                    #focus section
                    child_id = self.treeMsg.get_children()[-1]
                    self.treeMsg.focus(child_id)
                    self.treeMsg.selection_set(child_id)
            else:
                for i in range(msgs_num):
                    if len(self.treeMsg.get_children()) == MAX_DISPLAY:
                        self.treeMsg.delete(self.treeMsg.get_children()[0])
                    self.treeMsg.insert('', 'end', values=self.CANMsg2View(msgs[i].frame, is_send))
                    #focus section
                    child_id = self.treeMsg.get_children()[-1]
                    self.treeMsg.focus(child_id)
                    self.treeMsg.selection_set(child_id)

    def PeriodSendIdUpdate(self, is_ext):
        self._cur_id += 1
        if is_ext:
            if self._cur_id > 0x1FFFFFFF:
                self._cur_id = 0
        else:
            if self._cur_id > 0x7FF:
                self._cur_id = 0

    def PeriodSendComplete(self):
        self._is_sending = False
        self.strvSend.set("发送")
        self._send_thread.send_stop()

    def PeriodSend(self):
        if self._is_canfd_msg: 
            ret = self._zcan.TransmitFD(self._can_handle, self._send_msgs, self._send_num)
        else:
            ret = self._zcan.Transmit(self._can_handle, self._send_msgs, self._send_num)
        
        #update transmit display
        self._tx_cnt += ret
        self.strvTxCnt.set(str(self._tx_cnt))
        self.ViewDataUpdate(self._send_msgs, ret, self._is_canfd_msg, True)
        
        if ret != self._send_num:
            self.PeriodSendComplete()
            messagebox.showerror(title="发送报文", message="发送失败！")
            return

        self._send_cnt -= 1
        if self._send_cnt:
            if self._id_increase:
                for i in range(self._send_num):
                    self._send_msgs[i].frame.can_id = self._cur_id
                    self.PeriodSendIdUpdate(self._send_msgs[i].frame.eff)
        else:
            self.PeriodSendComplete()

    def MsgSend(self, msg, is_canfd, num=1, cnt=1, period=0, id_increase=0):
        self._id_increase  = id_increase
        self._send_num     = num if num else 1
        self._send_cnt     = cnt if cnt else 1
        self._is_canfd_msg = is_canfd

        if is_canfd:    
            self._send_msgs = (ZCAN_TransmitFD_Data * self._send_num)()
        else:
            self._send_msgs = (ZCAN_Transmit_Data * self._send_num)()

        self._cur_id = msg.frame.can_id
        for i in range(self._send_num):
            self._send_msgs[i] = msg
            self._send_msgs[i].frame.can_id = self._cur_id
            self.PeriodSendIdUpdate(self._send_msgs[i].frame.eff)

        self._is_sending = True    
        self._send_thread.send_start(period * 0.001)
    def DevInfoRead(self):
        info = self._zcan.GetDeviceInf(self._dev_handle)
        if info != None:
            self.strvHwVer.set(info.hw_version)
            self.strvFwVer.set(info.fw_version)
            self.strvDrVer.set(info.dr_version)
            self.strvInVer.set(info.in_version)
            self.strvCANNum.set(str(info.can_num))
            self.strvSerial.set(info.serial)
            self.strvHwType.set(info.hw_type)

    def DevInfoClear(self):
        self.strvHwVer.set('')
        self.strvFwVer.set('')
        self.strvDrVer.set('')
        self.strvInVer.set('')
        self.strvCANNum.set('')
        self.strvSerial.set('')
        self.strvHwType.set('')
###############################################################################
### Event handers
###############################################################################
    def Form_OnClosing(self):
        if self._isOpen:
            self.btnDevCtrl.invoke()

        self.destroy()

    def BtnOpenDev_Click(self):
        if self._isOpen:
            #Close Channel 
            if self._isChnOpen:
                self.btnCANCtrl.invoke()

            #Close Device
            self._zcan.CloseDevice(self._dev_handle)

            self.DevInfoClear()
            self.strvDevCtrl.set("打开")
            self.cmbDevType["state"] = "readonly"
            self.cmbDevIdx["state"] = "readonly"
            self._isOpen = False
        else:
            self._cur_dev_info = self._dev_info[self.cmbDevType.get()]

            #Open Device
            self._dev_handle = self._zcan.OpenDevice(self._cur_dev_info["dev_type"], 
                                                     self.cmbDevIdx.current(), 0)
            if self._dev_handle == INVALID_DEVICE_HANDLE:
                #Open failed
                messagebox.showerror(title="打开设备", message="打开设备失败！")
                return 
            
            #Update Device Info Display
            self.DevInfoRead()

            self._is_canfd = self._cur_dev_info["chn_info"]["is_canfd"]
            self._res_support = self._cur_dev_info["chn_info"]["sf_res"]
            if self._is_canfd:
                self.cmbMsgCANFD["value"] = ("CAN", "CANFD", "CANFD BRS")
            else:
                self.cmbMsgCANFD["value"] = ("CAN")

            self.strvDevCtrl.set("关闭")
            self.cmbDevType["state"] = tk.DISABLED
            self.cmbDevIdx["state"] = tk.DISABLED
            self._isOpen = True 
        self.ChnInfoUpdate(self._isOpen)
        self.ChnInfoDisplay(self._isOpen)

    def BtnOpenCAN_Click(self):
        if self._isChnOpen:
            #wait read_thread exit
            self._terminated = True
            self._read_thread.join(0.1)

            #stop send thread
            self._send_thread.stop()

            #cancel send
            if self._is_sending:
                self.btnMsgSend.invoke()

            #Close channel
            self._zcan.ResetCAN(self._can_handle)
            self.strvCANCtrl.set("打开")
            self._isChnOpen = False
            self.btnMsgSend["state"] = tk.DISABLED
        else:
            #Initial channel
            if self._res_support: #resistance enable
                ip = self._zcan.GetIProperty(self._dev_handle)
                self._zcan.SetValue(ip, 
                                    str(self.cmbCANChn.current()) + "/initenal_resistance", 
                                    '1' if self.cmbResEnable.current() == 0 else '0')
                self._zcan.ReleaseIProperty(ip)

            #set usbcan-e-u baudrate
            if self._cur_dev_info["dev_type"] in USBCAN_XE_U_TYPE:
                ip = self._zcan.GetIProperty(self._dev_handle)
                self._zcan.SetValue(ip, 
                                    str(self.cmbCANChn.current()) + "/baud_rate", 
                                    self._cur_dev_info["chn_info"]["baudrate"][self.cmbBaudrate.get()])
                self._zcan.ReleaseIProperty(ip)

            #set usbcanfd clock 
            if self._cur_dev_info["dev_type"] in USBCANFD_TYPE:
                ip = self._zcan.GetIProperty(self._dev_handle)
                self._zcan.SetValue(ip, str(self.cmbCANChn.current()) + "/clock", "60000000")
                self._zcan.ReleaseIProperty(ip)
            
            chn_cfg = ZCAN_CHANNEL_INIT_CONFIG()
            chn_cfg.can_type = ZCAN_TYPE_CANFD if self._is_canfd else ZCAN_TYPE_CAN
            if self._is_canfd:
                chn_cfg.config.canfd.mode = self.cmbCANMode.current()
                chn_cfg.config.canfd.abit_timing = self._cur_dev_info["chn_info"]["baudrate"][self.cmbBaudrate.get()]
                chn_cfg.config.canfd.dbit_timing = self._cur_dev_info["chn_info"]["data_baudrate"][self.cmbDataBaudrate.get()]
            else:
                chn_cfg.config.can.mode = self.cmbCANMode.current()
                if self._cur_dev_info["dev_type"] in USBCAN_I_II_TYPE:
                    brt = self._cur_dev_info["chn_info"]["baudrate"][self.cmbBaudrate.get()]
                    chn_cfg.config.can.timing0 = brt["timing0"] 
                    chn_cfg.config.can.timing1 = brt["timing1"]
                    chn_cfg.config.can.acc_code = 0
                    chn_cfg.config.can.acc_mask = 0xFFFFFFFF

            self._can_handle = self._zcan.InitCAN(self._dev_handle, self.cmbCANChn.current(), chn_cfg)
            if self._can_handle == INVALID_CHANNEL_HANDLE:
                messagebox.showerror(title="打开通道", message="初始化通道失败!")
                return 

            ret = self._zcan.StartCAN(self._can_handle)
            if ret != ZCAN_STATUS_OK: 
                messagebox.showerror(title="打开通道", message="打开通道失败!")
                return 

            #start send thread
            self._send_thread = PeriodSendThread(self.PeriodSend)
            self._send_thread.start()

            #start receive thread
            self._terminated = False
            self._read_thread = threading.Thread(None, target=self.MsgReadThreadFunc)
            self._read_thread.start()

            self.strvCANCtrl.set("关闭")
            self._isChnOpen = True 
            self.btnMsgSend["state"] = tk.NORMAL
        self.ChnInfoDisplay(not self._isChnOpen)

    def BtnClrCnt_Click(self):
        self._tx_cnt = 0
        self._rx_cnt = 0
        self._view_cnt = 0
        self.strvTxCnt.set("0")
        self.strvRxCnt.set("0")
        # self.treeMsg
        for item in self.treeMsg.get_children():
            self.treeMsg.delete(item)

    def CmbMsgFormatUpdate(self, *args):
        if self.cmbMsgFormat.current() == 0: #Data Frame
            if self._is_canfd:
                self.cmbMsgCANFD["value"] = ("CAN", "CANFD", "CANFD BRS")
            else:
                self.cmbMsgCANFD["value"] = ("CAN")
                self.cmbMsgCANFD.current(0)
        else: #Remote Frame
            self.cmbMsgCANFD["value"] = ("CAN")
            self.cmbMsgCANFD.current(0)

    def CmbMsgCANFDUpdate(self, *args):
        tmp = self.cmbMsgLen.current()
        self.cmbMsgLen["value"] = tuple([self.__dlc2len(i) for i in range(16 if self.cmbMsgCANFD.current() else 9)]) 
        if tmp >= len(self.cmbMsgLen["value"]):
            self.cmbMsgLen.current(len(self.cmbMsgLen["value"]) - 1)            

    def BtnSendMsg_Click(self): 
        if not self._is_sending:
            is_canfd_msg = True if self.cmbMsgCANFD.current() > 0 else False
            if is_canfd_msg:
                msg = ZCAN_TransmitFD_Data()
            else:
                msg = ZCAN_Transmit_Data()

            msg.transmit_type = self.cmbSendType.current()
            try:
                msg.frame.can_id = int(self.entryMsgID.get(), 16)
            except:
                msg.frame.can_id = 0
            msg.frame.rtr = self.cmbMsgFormat.current()
            msg.frame.eff = self.cmbMsgType.current()

            if not is_canfd_msg:
                msg.frame.can_dlc = self.cmbMsgLen.current()
                msg_len = msg.frame.can_dlc
            else:
                msg.frame.brs = 1 if self.cmbMsgCANFD.current() == 2 else 0
                msg.frame.len = self.__dlc2len(self.cmbMsgLen.current())
                msg_len = msg.frame.len

            data = self.entryMsgData.get().split(' ')
            for i in range(msg_len):
                if i < len(data):
                    try:
                        msg.frame.data[i] = int(data[i], 16)
                    except:
                        msg.frame.data[i] = 0
                else:
                    msg.frame.data[i] = 0

            try:
                msg_num = int(self.entryMsgNum.get())
                msg_cnt = int(self.entryMsgCnt.get())
                period  = int(self.entryMsgPeriod.get())
            except:
                msg_num = 1
                msg_cnt = 1
                period  = 1
            self.MsgSend(msg, is_canfd_msg, msg_num, msg_cnt, period, self.varIDInc.get())
            self.strvSend.set("停止发送")
        else:
            self.PeriodSendComplete()

if __name__ == "__main__":
    demo = ZCAN_Demo()
    demo.mainloop()


import time

from devices.zlg.zlgcan import ZCAN, ZCAN_CHANNEL_INIT_CONFIG, ZCAN_TYPE_CAN, ZCAN_Transmit_Data

if __name__ == '__main__':
    _zcan = ZCAN()
    _dev_handle = _zcan.OpenDevice(device_type=4, device_index=0, reserved=0)
    print("_device_handle: ", _dev_handle)

    chn_cfg = ZCAN_CHANNEL_INIT_CONFIG()
    chn_cfg.can_type = ZCAN_TYPE_CAN
    chn_cfg.config.can.timing0 = 0
    chn_cfg.config.can.timing1 = 0x1C
    chn_cfg.config.can.acc_code = 0
    chn_cfg.config.can.acc_mask = 0xFFFFFFFF

    _can_handle_0 = _zcan.InitCAN(device_handle=_dev_handle, can_index=0, init_config=chn_cfg)
    print("_can_handle_0: ", _can_handle_0)

    ret = _zcan.StartCAN(_can_handle_0)
    print("_can_handle_0 StartCAN ret: (1是OK)", ret)

    _can_handle_1 = _zcan.InitCAN(device_handle=_dev_handle, can_index=1, init_config=chn_cfg)
    print("_can_handle_1: ", _can_handle_1)

    ret = _zcan.StartCAN(_can_handle_1)
    print("_can_handle_1 StartCAN ret: (1是OK)", ret)



    msg = ZCAN_Transmit_Data()
    msg.frame.can_id = 0x100
    msg.frame.rtr = 0
    msg.frame.eff = 0
    msg.frame.can_dlc = 8
    msg.frame.data[0] = 1
    msg.frame.data[1] = 1
    msg.frame.data[2] = 2
    msg.frame.data[3] = 3
    msg.frame.data[4] = 5
    msg.frame.data[5] = 8
    msg.frame.data[6] = 0xA
    msg.frame.data[7] = 0xF

    _send_num = 1
    _send_msgs = (ZCAN_Transmit_Data * _send_num)()
    _send_msgs[0] = msg

    trans_success_count = _zcan.Transmit(_can_handle_0, _send_msgs, _send_num)
    print("trans_success_count: ", trans_success_count)

    rec_num = _zcan.GetReceiveNum(_can_handle_1, ZCAN_TYPE_CAN)
    print("rec_num: ", rec_num)

    MAX_RCV_NUM = 10
    read_cnt = MAX_RCV_NUM if rec_num >= MAX_RCV_NUM else rec_num
    receive_msgs, act_num = _zcan.Receive(_can_handle_1, read_cnt)
    msg = receive_msgs[0].frame
    readable_data = ''.join(hex(msg.data[i])[2:] + ' ' for i in range(msg.can_dlc))
    print("readable_data: ", readable_data, " act_num: ", act_num)




    print("After 3s, close chn automatically")
    time.sleep(3)

    ret = _zcan.ResetCAN(_can_handle_0)
    print("_can_handle_0 ResetCAN ret: (1是OK)", ret)

    ret = _zcan.ResetCAN(_can_handle_1)
    print("_can_handle_1 ResetCAN ret: (1是OK)", ret)

    ret = _zcan.CloseDevice(_dev_handle)
    print("CloseDevice ret: (1是OK)", ret)




class Signal:

    def __init__(self, dbc, name=None, curr_value=None):
        if not dbc.check_signal_name(name):
            raise
        choices = dbc.find_choices_by_signal_name(name)
        if choices is not None:
            if curr_value not in choices.keys():
                raise
        self.name = name
        self.curr_value = curr_value
        self.value_to_choice = choices
        self.msg_id = dbc.find_msg_id_by_signal_name(name)
        self.dbc = dbc

    def __repr__(self):
        return "Signal(msg_id: %s, name: %s, curr_value: %s, value_to_choice: %s)" \
               % (self.msg_id, self.name, self.curr_value, self.value_to_choice)

    def __eq__(self, other):
        if other is None:
            return False
        return self.name == other.name and self.curr_value == other.curr_value

    def __hash__(self):
        return hash(self.name) + hash(self.curr_value)


class SignalGroup:

    def __init__(self, dbc, *sigs):
        self._missing_signal_init_flag = False
        self.dbc = dbc
        self.msg_id = None
        self.signals = dict()
        self.interval = None
        if sigs is not None:
            for sig in sigs:
                self.update(sig)

    def init_missing_sig_as_default_value(self):
        # we add missing signal value to default 0,
        # event if we call add after creation of signal_group, new signal push out the old one
        for sig_name in self.dbc.find_signal_names_by_msg_id(self.msg_id):
            if self.signals.get(sig_name) is None:
                self.update(Signal(self.dbc, sig_name, 0))

    def update(self, signal):
        if signal is None:
            raise
        if len(self.signals) > 0 and signal.msg_id != self.msg_id:
            raise
        self.signals[signal.name] = signal

        if not self._missing_signal_init_flag:
            self.msg_id = signal.msg_id
            self.interval = self.dbc.find_msg_interval_by_msg_id(self.msg_id)
            self.init_missing_sig_as_default_value()
            self._missing_signal_init_flag = True


    def update_signal_value(self, name, curr_value):
        self.update(Signal(self.dbc, name, curr_value))

    def clone(self):
        return SignalGroup(self. dbc, *self.signals.values())

    def to_data(self):
        signal_name_to_value = {sig.name: sig.curr_value for sig in self.signals.values()}
        data_bytes = self.dbc.encode_message(self.msg_id, signal_name_to_value)
        return list(data_bytes)

    def to_readable_data(self):
        return self.dbc.decode_message(self.msg_id, self.to_data(), True)

    def items(self):
        return self.signals.items()

    def __repr__(self):
        return "SignalGroup(msg_id: %s, interval: %s, signals: %s)" \
               % (self.msg_id, self.interval, self.signals)




from unittest import TestCase, TestSuite, TextTestRunner

import cantools

from framework.candb import DBC
from framework.canfactory import create_batch_from_signal_group
from framework.cansignal import Signal, SignalGroup


class TestCanSignal(TestCase):

    def setUp(self):
        self.dbc = DBC()
        pdcu_db = cantools.db.load_file('../file/pdcu.dbc', database_format='dbc')
        ddcu_db = cantools.db.load_file('../file/ddcu.dbc', database_format='dbc')
        self.dbc.load(pdcu_db)
        self.dbc.load(ddcu_db)

    def tearDown(self):
        pass

    def test_001_create_signal(self):
        sig_name = "PDCU_MemoryMode"
        sig = Signal(dbc=self.dbc,
                     name=sig_name,
                     curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        print(sig)

    def test_002_create_signal_group(self):
        sig1 = Signal(dbc=self.dbc,
                     name="PDCU_MemoryMode",
                     curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc,
                     name="PDCU_MemoryResult",
                     curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        print(sg)

    def test_003_signal_group_to_data(self):
        sig1 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryMode",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryResult",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        print(sg.to_data())

    def test_003_signal_group_update(self):
        sig1 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryMode",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryResult",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        sg.update_signal_value("PDCU_MemoryResult", self.dbc.PDCU_2.PDCU_MemoryResult.Succeed)
        print(sg.to_data())
        print(sg.to_readable_data())

    def test_004_signal_group_clone(self):
        sig1 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryMode",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryResult",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        new_sg = sg.clone()
        new_sg.update_signal_value("PDCU_MemoryResult", self.dbc.PDCU_2.PDCU_MemoryResult.Succeed)
        print("sg: ", sg.to_readable_data())
        print("new_sg", new_sg.to_readable_data())

    def test_005_signal_group_to_batch(self):
        sig1 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryMode",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc,
                      name="PDCU_MemoryResult",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        batch = create_batch_from_signal_group(sg)
        print(batch)


if __name__ == '__main__':
    suit = TestSuite()
    suit.addTest(TestCanSignal())
    runner = TextTestRunner()
    runner.run(suit)


class CanMsg:

    def __init__(self, id, data, count=1, interval=0):
        self.id = id
        self.data = data
        self.count = count
        self.interval = interval

    def __repr__(self):
        return "CanMsg(id: %s, data: %s, count: %s, interval: %s)" \
               % (hex(self.id), self.data, self.count, self.interval)

    def __eq__(self, other):
        if other is None:
            return False
        return self.id == other.id and \
               self.data == other.data and \
               self.count == other.count and \
               self.interval == other.interval

    def __hash__(self):
        return self.id * 100 + self.count * 10 + self.interval

    def same_content(self, other):
        if other is None:
            return False
        return self.id == other.id and \
               self.data == other.data


class CanMsgBatch:

    def __init__(self, can_msg=None, loop_count=-1, loop_interval=0):
        self.can_msgs = [] if can_msg is None else [can_msg]
        self.loop_count = loop_count
        self.loop_interval = loop_interval

    def add(self, can_msg):
        self.can_msgs.append(can_msg)

    def __repr__(self):
        return "CanMsgBatch(can_msgs: %s, loop_count: %s, loop_interval: %s)" \
               % (self.can_msgs, self.loop_count, self.loop_interval)

    def __eq__(self, other):
        if other is None:
            return False
        return self.can_msgs == other.can_msgs and \
               self.loop_count == other.loop_count and \
               self.loop_interval == other.loop_interval

    def __hash__(self):
        return len(self.can_msgs) * 100 + self.loop_count * 10 + self.loop_interval


from concurrent.futures.thread import ThreadPoolExecutor

from framework.concurrentcanbox import BatchSendWorker, BatchReceiveWorker, KeyWorker


class Bus:

    def __init__(self, device):
        self.start_flag = False
        self.device = device
        self.node_list = []
        self.executor = ThreadPoolExecutor()
        self.send_worker = BatchSendWorker(device=device, bus=self)
        self.receive_worker = BatchReceiveWorker(device=device, bus=self)
        self.key_worker = KeyWorker(bus=self)

    def start(self):
        self.device.open()
        self.device.open_channel()
        self.receive_worker.receive()
        # self.key_worker.start_listening()
        self.start_flag = True

    def stop(self):
        self.device.close_channel()
        self.device.close()
        self.receive_worker.stop_receiving()
        # self.key_worker.stop_listening()
        self.start_flag = False

    def register(self, node):
        if self.start_flag:
            raise RuntimeError("Can not register node after but called start")
        if node is not None:
            self.node_list.append(node)
            node.set_send_worker(self.send_worker)

    def _broadcast_msg(self, can_msg):
        for node in self.node_list:
            if node.name != can_msg.control.send_from:
                node.receive_msg(can_msg)

    def broadcast_msg(self, can_msg):
        self.executor.submit(self._broadcast_msg, can_msg)

    def _broadcast_key_event(self, key_event):
        for node in self.node_list:
            node.receive_key_event(key_event)

    def broadcast_key_event(self, key_event):
        self.executor.submit(self._broadcast_key_event, key_event)


import unittest
from unittest.case import TestCase

import cantools

from devices.devices import DEVICE
from framework.canbus import Bus
from framework.candb import DBC
from framework.canfactory import create_batch_from_zcanpro_exporting_file, \
    create_batch_from_signal_group, create_can_msg_from_signal_group
from framework.cansignal import Signal, SignalGroup
from framework.nodes import VirtualNode


class TestVirtualNode(TestCase):

    def setUp(self):
        dbc = DBC()
        bus = Bus(DEVICE)
        pdcu_db = cantools.db.load_file('../file/pdcu.dbc', database_format='dbc')
        ddcu_db = cantools.db.load_file('../file/ddcu.dbc', database_format='dbc')
        dbc.load(pdcu_db)
        dbc.load(ddcu_db)
        ddcu = VirtualNode("DriverDCU", dbc)
        pdcu = VirtualNode("PassengerDCU", dbc)
        dscu = VirtualNode("DSCU", dbc)
        logger = VirtualNode("logger", dbc)
        logger.on_receive_any_msg().print()
        bus.register(ddcu)
        bus.register(pdcu)
        bus.register(dscu)
        bus.register(logger)

        self.dbc = dbc
        self.pdcu = pdcu
        self.ddcu = ddcu
        self.dscu = dscu
        self.bus = bus
        pass

    def tearDown(self):
        pass

    def test_001_check_id(self):
        self.assertTrue(self.ddcu._check_send_id(0x1E6))
        self.assertEqual(self.pdcu.id_set(), {0x1E8, 0x1EA})

    def test_002_send_and_receive(self):
        self.bus.start()

        batch = create_batch_from_zcanpro_exporting_file("files/DDCU_1.list")
        handle_1 = self.ddcu.send(batch)
        batch = create_batch_from_zcanpro_exporting_file("files/PDCU_1.list")
        handle_2 = self.pdcu.send(batch)

        handle_1.after(2).stop_sending()
        handle_2.after(1).stop_sending()

        self.bus.stop()

    def test_003_trigger_on_receive_msg(self):
        self.bus.start()
        sig1 = Signal(dbc=self.dbc, name="DSCU_MemoryMode",
                      curr_value=self.dbc.DSCU_Cmd.DSCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc, name="DSCU_MemoryResult",
                      curr_value=self.dbc.DSCU_Cmd.DSCU_MemoryResult.Succeed)
        sg = SignalGroup(self.dbc, sig1, sig2)

        can_msg = create_can_msg_from_signal_group(sg)
        self.pdcu.on_receive_msg(can_msg).print()

        # sg = sg.clone()
        # sg.update_signal_value("DSCU_MemoryResult", self.dbc.DSCU_Cmd.DSCU_MemoryResult.Succeed)
        batch = create_batch_from_signal_group(sg)
        self.dscu.send(batch).after(1)

        self.bus.stop()

    def test_005_trigger(self):
        self.bus.start()
        sig1 = Signal(dbc=self.dbc, name="DSCU_MemoryMode",
                      curr_value=self.dbc.DSCU_Cmd.DSCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc, name="DSCU_MemoryResult",
                      curr_value=self.dbc.DSCU_Cmd.DSCU_MemoryResult.Succeed)
        sg = SignalGroup(self.dbc, sig1, sig2)

        batch = create_batch_from_signal_group(sg)
        self.dscu\
            .on_receive_signal(name="PDCU_MemoryResult", value=self.dbc.PDCU_2.PDCU_MemoryResult.Succeed)\
            .send(batch)

        sig1 = Signal(dbc=self.dbc, name="PDCU_MemoryMode",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryMode.Read)
        sig2 = Signal(dbc=self.dbc, name="PDCU_MemoryResult",
                      curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Invalid_value)
        sg = SignalGroup(self.dbc, sig1, sig2)
        batch = create_batch_from_signal_group(sg)
        self.pdcu.send(batch).after(1)

        sg = sg.clone()
        sg.update_signal_value(
            name="PDCU_MemoryResult",
            curr_value=self.dbc.PDCU_2.PDCU_MemoryResult.Succeed)

        batch = create_batch_from_signal_group(sg)
        self.pdcu.send(batch).after(1)

        self.bus.stop()


if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(TestVirtualNode())

    runner = unittest.TextTestRunner()
    runner.run(suite)


import time
import unittest
from unittest import TestCase

from devices.devices import ZlgDevice
from framework.canbox import CanBox
from framework.canfactory import create_batch_from_zcanpro_exporting_file
from framework.canmsg import CanMsgBatch, CanMsg


class TestCanBox(TestCase):

    def setUp(self):
        dev = ZlgDevice()
        self.can_box = CanBox(dev)

    def tearDown(self):
        pass

    def test_open_and_close_device(self):
        self.assertTrue(self.can_box.open())
        self.assertTrue(self.can_box.close())

    def test_open_and_close_channel(self):
        self.assertTrue(self.can_box.open())
        self.assertTrue(self.can_box.open_channel())
        self.assertTrue(self.can_box.close_channel())
        self.assertTrue(self.can_box.close())

    def test_device_online(self):
        self.assertTrue(self.can_box.open())
        self.assertTrue(self.can_box.is_device_online())
        self.assertTrue(self.can_box.close())
        self.assertFalse(self.can_box.is_device_online())

    def test_send_batch_1(self):
        def on_receive(rec_batch, act_cnt):
            mock_batch = CanMsgBatch(CanMsg(486, [0, 12, 0]))
            self.assertEqual(1, act_cnt)
            self.assertEqual(mock_batch, rec_batch)

        self.assertTrue(self.can_box.open(on_receive))
        self.assertTrue(self.can_box.open_channel())
        self.can_box.send_batch_to_myself(create_batch_from_zcanpro_exporting_file("files/test_batch_1_msg.list"))
        self.assertTrue(self.can_box.close_channel())
        self.assertTrue(self.can_box.close())

    def test_send_batch_2(self):
        def on_receive(rec_batch, act_cnt):
            m1 = CanMsg(486, [0, 12, 0])
            m2 = CanMsg(464, [8, 0, 0, 0, 0, 0, 0, 0])
            m3 = CanMsg(514, [0, 0, 0, 0, 0, 0, 0, 0])
            ms = [m1, m2, m3]
            mock_batch = CanMsgBatch()
            mock_batch.can_msg = ms
            # self.assertEqual(mock_batch, rec_batch)
            # self.assertEqual(1, act_cnt)
            # print(rec_batch)

        self.assertTrue(self.can_box.open(on_receive))
        self.assertTrue(self.can_box.open_channel())
        self.can_box.send_batch_to_myself(create_batch_from_zcanpro_exporting_file("files/test_batch_3_msgs.list"))
        time.sleep(5)
        self.assertTrue(self.can_box.close_channel())
        self.assertTrue(self.can_box.close())


if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(TestCanBox())

    runner = unittest.TextTestRunner()
    runner.run(suite)



import time
from abc import abstractmethod
from datetime import datetime

import keyboard


class SendOperation:

    def __init__(self, node):
        self.node = node

    def send(self, batch):
        self.node.send(batch)


class PrintOperation:

    def __init__(self, node):
        self.node = node

    def print(self, obj):
        t = datetime.now().strftime('%H:%M:%S.%f')[:-3]
        print("[%s] %s #print: %s" % (t, self.node.name, obj))


class PrintReadableMsgOperation:

    def __init__(self, node):
        self.node = node
        self.dbc = node.dbc

    def print_msg(self, can_msg):
        readable_msg = self.dbc.decode_message(can_msg.id, can_msg.data, True)
        t = datetime.now().strftime('%H:%M:%S.%f')[:-3]
        print("(%s) %s #print_msg: %s" % (t, self.node.name, readable_msg))


class WriteAscFileOperation:

    def __init__(self, node, file_path):
        self.node = node
        self.file_path = file_path
        self.start_time = None
        with open(self.file_path, "w") as f:
            f.write("")

    def write_asc_file(self, can_msg):
        if self.start_time is None:
            self.start_time = time.time()
        delta_time = time.time() - self.start_time
        can_id = hex(can_msg.id).replace("0x", "")
        dlc = len(can_msg.data)
        data = can_msg.data
        data_str = ""
        for d in data:
            data_str += str(hex(d)).replace("0x", "").zfill(2)
            data_str += " "
        asc_line = "%s 1 %s Rx d %s %s\n" % (format(delta_time, '.6f'), can_id, dlc, data_str)
        with open(self.file_path, "a") as f:
            print(asc_line)
            f.write(asc_line)


class FunctionOperation:

    def __init__(self, node, fun, *args):
        self.node = node
        self.fun = fun
        self.args = args

    def run(self):
        self.fun(*self.args)


class Trigger:

    def __init__(self, node):
        self.node = node
        self.to_do = None
        self.print_operation = None
        self.print_readable_msg_operation = None
        self.write_asc_file_operation = None
        self.send_operation = None
        self.function_operation = None

    @abstractmethod
    def check(self, can_msg):
        return True

    def do_it(self, obj):
        if self.to_do is not None:
            self.to_do(obj)

    def print(self):
        if self.print_operation is None:
            self.print_operation = PrintOperation(self.node)
        self.to_do = lambda obj: self.print_operation.print(obj)

    def print_msg(self):
        if self.print_readable_msg_operation is None:
            self.print_readable_msg_operation = PrintReadableMsgOperation(self.node)
        self.to_do = lambda can_msg: self.print_readable_msg_operation.print_msg(can_msg)

    def write_asc_file(self, file_path):
        if self.write_asc_file_operation is None:
            self.write_asc_file_operation = WriteAscFileOperation(self.node, file_path)
        self.to_do = lambda can_msg: self.write_asc_file_operation.write_asc_file(can_msg)

    def send(self, batch):
        if self.send_operation is None:
            self.send_operation = SendOperation(self.node)
        self.to_do = lambda not_used: self.send_operation.send(batch)

    def run(self, fun, *args):
        if self.function_operation is None:
            self.function_operation = FunctionOperation(self.node, fun, *args)
        self.to_do = lambda not_used: self.function_operation.run()


class OnReceiveAnyMsgTrigger(Trigger):

    def __init__(self, node):
        super().__init__(node)

    def check(self, can_msg):
        return True


class OnReceiveMsgExcludeIdsTrigger(Trigger):

    def __init__(self, node, exclude_ids):
        super().__init__(node)
        self.exclude_ids = exclude_ids

    def check(self, can_msg):
        return can_msg.id not in self.exclude_ids


class OnReceiveMsgIdsTrigger(Trigger):

    def __init__(self, node, ids):
        super().__init__(node)
        self.ids = ids

    def check(self, can_msg):
        return can_msg.id in self.ids


class OnReceiveMsgTrigger(Trigger):

    def __init__(self, node, can_msg):
        super().__init__(node)
        self.cared_can_msg = can_msg

    def check(self, can_msg):
        return self.cared_can_msg.same_content(can_msg)


class OnReceiveSignalTrigger(Trigger):

    def __init__(self, node, signal):
        super().__init__(node)
        self.signal = signal

    def check(self, can_msg):
        if can_msg.id == self.signal.msg_id:
            dbc = self.signal.dbc
            signal_name_to_value = dbc.decode_message(can_msg.id, can_msg.data)
            value = signal_name_to_value.get(self.signal.name)
            if value is None:
                raise
            if value == self.signal.curr_value:
                return True
        return False


class OnReceiveSignalGroupTrigger(Trigger):

    def __init__(self, node, signal_group):
        super().__init__(node)
        self.signal_group = signal_group

    def check(self, can_msg):
        if can_msg.id == self.signal_group.msg_id:
            dbc = self.signal_group.dbc
            signal_name_to_value = dbc.decode_message(can_msg.id, can_msg.data)
            for sig_name, sig in self.signal_group.items():
                if signal_name_to_value.get(sig_name) is None:
                    raise
                # print("sig_name: ", sig_name, "signal_name_to_value.get(sig_name): ", signal_name_to_value.get(sig_name), "sig.curr_value: ", sig.curr_value)
                if signal_name_to_value.get(sig_name) != sig.curr_value:
                    return False
            return True
        return False


class OnKeyPressedTrigger(Trigger):

    def __init__(self, node, key_name):
        super().__init__(node)
        self.key_name = key_name

    def check(self, key_event):
        return key_event.event_type == keyboard.KEY_DOWN and key_event.name == self.key_name


class OnKeyReleaseTrigger(Trigger):

    def __init__(self, node, key_name):
        super().__init__(node)
        self.key_name = key_name

    def check(self, key_event):
        return key_event.event_type == keyboard.KEY_UP and key_event.name == self.key_name


from unittest.case import TestCase

import cantools

from framework.candb import DBC


class TestDBC(TestCase):

    def setUp(self):
        dbc = DBC()
        pdcu_db = cantools.db.load_file('../file/pdcu.dbc', database_format='dbc')
        ddcu_db = cantools.db.load_file('../file/ddcu.dbc', database_format='dbc')
        dbc.load(pdcu_db)
        dbc.load(ddcu_db)
        self.dbc = dbc
        self.pdcu_db = pdcu_db
        pass

    def tearDown(self):
        pass

    def test_001_load_two_dbc_files(self):
        dbc = self.dbc
        self.assertEqual("PDCU_2", dbc.PDCU_2.name)
        self.assertEqual("PDCU_MemoryMode", dbc.PDCU_2.PDCU_MemoryMode.name)
        self.assertEqual(0, dbc.PDCU_2.PDCU_MemoryMode.Memory)
        self.assertEqual(1, dbc.PDCU_2.PDCU_MemoryMode.Read)
        self.assertEqual(2, dbc.PDCU_2.PDCU_MemoryMode.Init)
        self.assertEqual(3, dbc.PDCU_2.PDCU_MemoryMode.No_action)

        self.assertEqual("DDCU_2", dbc.DDCU_2.name)
        self.assertEqual("DDCU_MemoryMode", dbc.DDCU_2.DDCU_MemoryMode.name)
        self.assertEqual(0, dbc.DDCU_2.DDCU_MemoryMode.Memory)
        self.assertEqual(1, dbc.DDCU_2.DDCU_MemoryMode.Read)
        self.assertEqual(2, dbc.DDCU_2.DDCU_MemoryMode.Init)
        self.assertEqual(3, dbc.DDCU_2.DDCU_MemoryMode.No_action)

    def test_002_decode(self):
        data = [0x21, 0x03, 0, 0, 0, 0, 0, 0]
        ret = self.pdcu_db.decode_message(frame_id_or_name=488, data=data, decode_choices=False)
        print(ret)

        ret_2 = self.pdcu_db.encode_message(frame_id_or_name=488, data=ret)
        # print(ret_2)
        self.assertEqual(list(ret_2), data)





import unittest
from unittest.case import TestCase

from devices.devices import Device
from framework.canfactory import create_batch_from_zcanpro_exporting_file
from framework.concurrentcanbox import BatchSendWorker


class TestMultiCanBox(TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_001_send_1_batch_10_loop(self):
        dev = Device()
        send_worker = BatchSendWorker(dev)
        batch = create_batch_from_zcanpro_exporting_file("files/test_multicanbox_1_msg_10_cnt.list")
        handle = send_worker.send(batch)
        handle.wait()

    def test_002_send_1_batch_infinity_loop(self):
        dev = Device()
        send_worker = BatchSendWorker(dev)
        batch = create_batch_from_zcanpro_exporting_file("files/test_multicanbox_1_msg_infinity_1.list")
        handle = send_worker.send(batch)
        handle.wait(5)
        handle.stop_sending()

    def test_003_send_2_batch(self):
        dev = Device()
        send_worker = BatchSendWorker(dev)
        handle_inf_1 = send_worker.send(create_batch_from_zcanpro_exporting_file("files/test_multicanbox_1_msg_infinity_1.list"))
        handle_inf_2 = send_worker.send(create_batch_from_zcanpro_exporting_file("files/test_multicanbox_1_msg_infinity_2.list"))
        handle_inf_1.wait(1)
        handle_inf_1.stop_sending()
        handle_inf_2.wait(3)

    def test_004_auto_stop_when_diff(self):
        dev = Device()
        send_worker = BatchSendWorker(dev)
        handle_ig_on = send_worker.send(
            create_batch_from_zcanpro_exporting_file("files/test_multicanbox_ig_on.list"),
            to_myself=False,
            auto_stop_sending_conflict_batch=False
        )
        handle_ig_on.after(1)

        handle_ig_off = send_worker.send(
            create_batch_from_zcanpro_exporting_file("files/test_multicanbox_ig_off.list"),
            to_myself=False,
            auto_stop_sending_conflict_batch=False)
        handle_ig_off.after(3).stop_sending()

if __name__ == '__main__':
    suite = unittest.TestSuite()
    suite.addTest(TestMultiCanBox())

    runner = unittest.TextTestRunner()
    runner.run(suite)
